@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -22,17 +22,17 @@ class IGUIObject;
 class CGUI;
 
 #include "ps/CStr.h"
-#include "ps/Overlay.h"
+#include "ps/Shapes.h"
 
 class GUITooltip
 {
 public:
 	GUITooltip();
-	void Update(IGUIObject* Nearest, CPos MousePos, CGUI* GUI);
+	void Update(IGUIObject* Nearest, const CPos& MousePos, CGUI* GUI);
 
 private:
 
-	void ShowTooltip(IGUIObject* obj, CPos pos, const CStr& style, CGUI* gui);
+	void ShowTooltip(IGUIObject* obj, const CPos& pos, const CStr& style, CGUI* gui);
 	void HideTooltip(const CStr& style, CGUI* gui);
 	bool GetTooltip(IGUIObject* obj, CStr& style);
 
