@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -21,7 +21,10 @@
 
 #include "gui/scripting/JSInterface_GUITypes.h"
 #include "gui/scripting/JSInterface_IGUIObject.h"
+
+#include "ps/GameSetup/Config.h"
 #include "ps/CLogger.h"
+#include "ps/Profile.h"
 #include "scriptinterface/ScriptInterface.h"
 
 
@@ -186,22 +189,16 @@ CPos IGUIObject::GetMousePos() const
 
 void IGUIObject::UpdateMouseOver(IGUIObject* const& pMouseOver)
 {
-	// Check if this is the object being hovered.
 	if (pMouseOver == this)
 	{
 		if (!m_MouseHovering)
-		{
-			// It wasn't hovering, so that must mean it just entered
 			SendEvent(GUIM_MOUSE_ENTER, "mouseenter");
-		}
 
-		// Either way, set to true
 		m_MouseHovering = true;
 
-		// call mouse over
 		SendEvent(GUIM_MOUSE_OVER, "mousemove");
 	}
-	else // Some other object (or none) is hovered
+	else
 	{
 		if (m_MouseHovering)
 		{
@@ -226,7 +223,6 @@ PSRETURN IGUIObject::SetSetting(const CStr& Setting, const CStrW& Value, const b
 	if (!SettingExists(Setting))
 		return PSRETURN_GUI_InvalidSetting;
 
-	// Get setting
 	SGUISetting set = m_Settings[Setting];
 
 #define TYPE(type) \
@@ -240,7 +236,6 @@ PSRETURN IGUIObject::SetSetting(const CStr& Setting, const CStrW& Value, const b
 
 	if (0)
 		;
-	// else...
 #include "GUItypes.h"
 #undef TYPE
 	else
@@ -419,11 +414,13 @@ void IGUIObject::RegisterScriptHandler(const CStr& Action, const CStr& Code, CGU
 	options.setFileAndLine(CodeName.c_str(), 0);
 	options.setCompileAndGo(true);
 
-	JS::RootedFunction func(cx, JS_CompileFunction(cx, globalObj,
-		buf, paramCount, paramNames, Code.c_str(), Code.length(), options));
-
-	if (!func)
-		return; // JS will report an error message
+	JS::RootedFunction func(cx);
+	JS::AutoObjectVector emptyScopeChain(cx);
+	if (!JS::CompileFunction(cx, emptyScopeChain, options, buf, paramCount, paramNames, Code.c_str(), Code.length(), &func))
+	{
+		LOGERROR("RegisterScriptHandler: Failed to compile the script for %s", Action.c_str());
+		return;
+	}
 
 	JS::RootedObject funcObj(cx, JS_GetFunctionObject(func));
 	SetScriptHandler(Action, funcObj);
@@ -507,9 +504,9 @@ JSObject* IGUIObject::GetJSObject()
 	// Cache the object when somebody first asks for it, because otherwise
 	// we end up doing far too much object allocation. TODO: Would be nice to
 	// not have these objects hang around forever using up memory, though.
-	if (m_JSObject.uninitialized())
+	if (!m_JSObject.initialized())
 	{
-		m_JSObject.set(cx, m_pGUI->GetScriptInterface()->CreateCustomObject("GUIObject"));
+		m_JSObject.init(cx, m_pGUI->GetScriptInterface()->CreateCustomObject("GUIObject"));
 		JS_SetPrivate(m_JSObject.get(), this);
 	}
 	return m_JSObject.get();
@@ -546,7 +543,7 @@ bool IGUIObject::IsRootObject() const
 void IGUIObject::TraceMember(JSTracer* trc)
 {
 	for (std::pair<const CStr, JS::Heap<JSObject*>>& handler : m_ScriptHandlers)
-		JS_CallHeapObjectTracer(trc, &handler.second, "IGUIObject::m_ScriptHandlers");
+		JS_CallObjectTracer(trc, &handler.second, "IGUIObject::m_ScriptHandlers");
 }
 
 PSRETURN IGUIObject::LogInvalidSettings(const CStr8& Setting) const
