@@ -49,7 +49,9 @@
 #include "ps/Globals.h"	// g_frequencyFilter
 #include "ps/Hotkey.h"
 #include "ps/ProfileViewer.h"
+#include "ps/Profile.h"
 #include "ps/Pyrogenesis.h"
+#include "ps/Replay.h"
 #include "ps/SavedGame.h"
 #include "ps/UserReport.h"
 #include "ps/World.h"
@@ -122,13 +124,11 @@ JS::Value GuiInterfaceCall(ScriptInterface::CxPrivate* pCxPrivate, const std::ws
 	if (!cmpGuiInterface)
 		return JS::UndefinedValue();
 
-	int player = g_Game->GetPlayerID();
-
 	JSContext* cxSim = sim->GetScriptInterface().GetContext();
 	JSAutoRequest rqSim(cxSim);
 	JS::RootedValue arg(cxSim, sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), data));
 	JS::RootedValue ret(cxSim);
-	cmpGuiInterface->ScriptCall(player, name, arg, &ret);
+	cmpGuiInterface->ScriptCall(g_Game->GetViewedPlayerID(), name, arg, &ret);
 
 	return pCxPrivate->pScriptInterface->CloneValueFromOtherContext(sim->GetScriptInterface(), ret);
 }
@@ -153,7 +153,7 @@ void PostNetworkCommand(ScriptInterface::CxPrivate* pCxPrivate, JS::HandleValue
 
 entity_id_t PickEntityAtPoint(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int x, int y)
 {
-	return EntitySelection::PickEntityAtPoint(*g_Game->GetSimulation2(), *g_Game->GetView()->GetCamera(), x, y, g_Game->GetPlayerID(), false);
+	return EntitySelection::PickEntityAtPoint(*g_Game->GetSimulation2(), *g_Game->GetView()->GetCamera(), x, y, g_Game->GetViewedPlayerID(), false);
 }
 
 std::vector<entity_id_t> PickPlayerEntitiesInRect(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int x0, int y0, int x1, int y1, int player)
@@ -186,7 +186,7 @@ std::vector<entity_id_t> PickNonGaiaEntitiesOnScreen(ScriptInterface::CxPrivate*
 
 std::vector<entity_id_t> PickSimilarPlayerEntities(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const std::string& templateName, bool includeOffScreen, bool matchRank, bool allowFoundations)
 {
-	return EntitySelection::PickSimilarEntities(*g_Game->GetSimulation2(), *g_Game->GetView()->GetCamera(), templateName, g_Game->GetPlayerID(), includeOffScreen, matchRank, false, allowFoundations);
+	return EntitySelection::PickSimilarEntities(*g_Game->GetSimulation2(), *g_Game->GetView()->GetCamera(), templateName, g_Game->GetViewedPlayerID(), includeOffScreen, matchRank, false, allowFoundations);
 }
 
 CFixedVector3D GetTerrainAtScreenPoint(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int x, int y)
@@ -204,7 +204,21 @@ std::wstring SetCursor(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const std
 
 bool IsVisualReplay(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 {
-	return g_Game ? g_Game->IsVisualReplay() : false;
+	if (!g_Game)
+		return false;
+
+	return g_Game->IsVisualReplay();
+}
+
+std::wstring GetCurrentReplayDirectory(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
+{
+	if (!g_Game)
+		return std::wstring();
+
+	if (g_Game->IsVisualReplay())
+		return OsPath(g_Game->GetReplayPath()).Parent().Filename().string();
+
+	return g_Game->GetReplayLogger().GetDirectory().Filename().string();
 }
 
 int GetPlayerID(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
@@ -222,10 +236,8 @@ void SetPlayerID(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int id)
 
 void SetViewedPlayer(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int id)
 {
-	if (!g_Game)
-		return;
-
-	g_Game->GetSimulation2()->GetSimContext().SetCurrentDisplayedPlayer(id);
+	if (g_Game)
+		g_Game->SetViewedPlayerID(id);
 }
 
 JS::Value GetEngineInfo(ScriptInterface::CxPrivate* pCxPrivate)
@@ -235,8 +247,8 @@ JS::Value GetEngineInfo(ScriptInterface::CxPrivate* pCxPrivate)
 
 void StartNetworkGame(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 {
-	ENSURE(g_NetServer);
-	g_NetServer->StartGame();
+	ENSURE(g_NetClient);
+	g_NetClient->SendStartGameMessage();
 }
 
 void StartGame(ScriptInterface::CxPrivate* pCxPrivate, JS::HandleValue attribs, int playerID)
@@ -293,7 +305,7 @@ JS::Value StartSavedGame(ScriptInterface::CxPrivate* pCxPrivate, const std::wstr
 		sim->GetScriptInterface().GetProperty(gameContextMetadata, "initAttributes", &gameInitAttributes);
 
 		int playerID;
-		sim->GetScriptInterface().GetProperty(gameContextMetadata, "player", playerID);
+		sim->GetScriptInterface().GetProperty(gameContextMetadata, "playerID", playerID);
 
 		// Start the game
 		g_Game->SetPlayerID(playerID);
@@ -306,37 +318,38 @@ JS::Value StartSavedGame(ScriptInterface::CxPrivate* pCxPrivate, const std::wstr
 void SaveGame(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring& filename, const std::wstring& description, JS::HandleValue GUIMetadata)
 {
 	shared_ptr<ScriptInterface::StructuredClone> GUIMetadataClone = pCxPrivate->pScriptInterface->WriteStructuredClone(GUIMetadata);
-	if (SavedGames::Save(filename, description, *g_Game->GetSimulation2(), GUIMetadataClone, g_Game->GetPlayerID()) < 0)
+	if (SavedGames::Save(filename, description, *g_Game->GetSimulation2(), GUIMetadataClone) < 0)
 		LOGERROR("Failed to save game");
 }
 
 void SaveGamePrefix(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring& prefix, const std::wstring& description, JS::HandleValue GUIMetadata)
 {
 	shared_ptr<ScriptInterface::StructuredClone> GUIMetadataClone = pCxPrivate->pScriptInterface->WriteStructuredClone(GUIMetadata);
-	if (SavedGames::SavePrefix(prefix, description, *g_Game->GetSimulation2(), GUIMetadataClone, g_Game->GetPlayerID()) < 0)
+	if (SavedGames::SavePrefix(prefix, description, *g_Game->GetSimulation2(), GUIMetadataClone) < 0)
 		LOGERROR("Failed to save game");
 }
 
 void SetNetworkGameAttributes(ScriptInterface::CxPrivate* pCxPrivate, JS::HandleValue attribs1)
 {
-	ENSURE(g_NetServer);
+	ENSURE(g_NetClient);
+
 	//TODO: This is a workaround because we need to pass a MutableHandle to a JSAPI functions somewhere
 	// (with no obvious reason).
 	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
 	JSAutoRequest rq(cx);
 	JS::RootedValue attribs(cx, attribs1);
 
-	g_NetServer->UpdateGameAttributes(&attribs, *(pCxPrivate->pScriptInterface));
+	g_NetClient->SendGameSetupMessage(&attribs, *(pCxPrivate->pScriptInterface));
 }
 
-void StartNetworkHost(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring& playerName)
+void StartNetworkHost(ScriptInterface::CxPrivate* pCxPrivate, const CStrW& playerName, const u16 serverPort)
 {
 	ENSURE(!g_NetClient);
 	ENSURE(!g_NetServer);
 	ENSURE(!g_Game);
 
 	g_NetServer = new CNetServer();
-	if (!g_NetServer->SetupConnection())
+	if (!g_NetServer->SetupConnection(serverPort))
 	{
 		pCxPrivate->pScriptInterface->ReportError("Failed to start server");
 		SAFE_DELETE(g_NetServer);
@@ -344,10 +357,10 @@ void StartNetworkHost(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring
 	}
 
 	g_Game = new CGame();
-	g_NetClient = new CNetClient(g_Game);
+	g_NetClient = new CNetClient(g_Game, true);
 	g_NetClient->SetUserName(playerName);
 
-	if (!g_NetClient->SetupConnection("127.0.0.1"))
+	if (!g_NetClient->SetupConnection("127.0.0.1", serverPort))
 	{
 		pCxPrivate->pScriptInterface->ReportError("Failed to connect to server");
 		SAFE_DELETE(g_NetClient);
@@ -355,16 +368,16 @@ void StartNetworkHost(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring
 	}
 }
 
-void StartNetworkJoin(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring& playerName, const std::string& serverAddress)
+void StartNetworkJoin(ScriptInterface::CxPrivate* pCxPrivate, const CStrW& playerName, const CStr& serverAddress, u16 serverPort)
 {
 	ENSURE(!g_NetClient);
 	ENSURE(!g_NetServer);
 	ENSURE(!g_Game);
 
 	g_Game = new CGame();
-	g_NetClient = new CNetClient(g_Game);
+	g_NetClient = new CNetClient(g_Game, false);
 	g_NetClient->SetUserName(playerName);
-	if (!g_NetClient->SetupConnection(serverAddress))
+	if (!g_NetClient->SetupConnection(serverAddress, serverPort))
 	{
 		pCxPrivate->pScriptInterface->ReportError("Failed to connect to server");
 		SAFE_DELETE(g_NetClient);
@@ -372,6 +385,11 @@ void StartNetworkJoin(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring
 	}
 }
 
+u16 GetDefaultPort(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
+{
+	return PS_DEFAULT_PORT;
+}
+
 void DisconnectNetworkGame(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 {
 	// TODO: we ought to do async reliable disconnections
@@ -384,17 +402,16 @@ void DisconnectNetworkGame(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 std::string GetPlayerGUID(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 {
 	if (!g_NetClient)
-		return "";
+		return std::string();
 
 	return g_NetClient->GetGUID();
 }
 
-bool KickPlayer(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const CStrW& playerName, bool ban)
+void KickPlayer(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const CStrW& playerName, bool ban)
 {
-	if (!g_NetServer)
-		return false;
+	ENSURE(g_NetClient);
 
-	return g_NetServer->KickPlayer(playerName, ban);
+	g_NetClient->SendKickPlayerMessage(playerName, ban);
 }
 
 JS::Value PollNetworkClient(ScriptInterface::CxPrivate* pCxPrivate)
@@ -412,23 +429,16 @@ JS::Value PollNetworkClient(ScriptInterface::CxPrivate* pCxPrivate)
 
 void AssignNetworkPlayer(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int playerID, const std::string& guid)
 {
-	ENSURE(g_NetServer);
+	ENSURE(g_NetClient);
 
-	g_NetServer->AssignPlayer(playerID, guid);
-}
-
-void SetNetworkPlayerStatus(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const std::string& guid, int ready)
-{
-	ENSURE(g_NetServer);
-
-	g_NetServer->SetPlayerReady(guid, ready);
+	g_NetClient->SendAssignPlayerMessage(playerID, guid);
 }
 
 void ClearAllPlayerReady (ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 {
-	ENSURE(g_NetServer);
+	ENSURE(g_NetClient);
 
-	g_NetServer->ClearAllPlayerReady();
+	g_NetClient->SendClearAllReadyMessage();
 }
 
 void SendNetworkChat(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const std::wstring& message)
@@ -445,13 +455,6 @@ void SendNetworkReady(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int messag
 	g_NetClient->SendReadyMessage(message);
 }
 
-void SendNetworkRejoined(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
-{
-	ENSURE(g_NetClient);
-
-	g_NetClient->SendRejoinedMessage();
-}
-
 JS::Value GetAIs(ScriptInterface::CxPrivate* pCxPrivate)
 {
 	return ICmpAIManager::GetAIs(*(pCxPrivate->pScriptInterface));
@@ -507,7 +510,7 @@ JS::Value LoadMapSettings(ScriptInterface::CxPrivate* pCxPrivate, const VfsPath&
 	return settings;
 }
 
-JS::Value GetMapSettings(ScriptInterface::CxPrivate* pCxPrivate)
+JS::Value GetInitAttributes(ScriptInterface::CxPrivate* pCxPrivate)
 {
 	if (!g_Game)
 		return JS::UndefinedValue();
@@ -515,11 +518,11 @@ JS::Value GetMapSettings(ScriptInterface::CxPrivate* pCxPrivate)
 	JSContext* cx = g_Game->GetSimulation2()->GetScriptInterface().GetContext();
 	JSAutoRequest rq(cx);
 
-	JS::RootedValue mapSettings(cx);
-	g_Game->GetSimulation2()->GetMapSettings(&mapSettings);
+	JS::RootedValue initAttribs(cx);
+	g_Game->GetSimulation2()->GetInitAttributes(&initAttribs);
 	return pCxPrivate->pScriptInterface->CloneValueFromOtherContext(
 		g_Game->GetSimulation2()->GetScriptInterface(),
-		mapSettings);
+		initAttribs);
 }
 
 /**
@@ -733,6 +736,12 @@ void Script_EndGame(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 	EndGame();
 }
 
+CStrW GetSystemUsername(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
+{
+	return sys_get_user_name();
+}
+
+
 // Cause the game to exit gracefully.
 // params:
 // returns:
@@ -757,7 +766,7 @@ bool IsPaused(ScriptInterface::CxPrivate* pCxPrivate)
 }
 
 // Pause/unpause the game
-void SetPaused(ScriptInterface::CxPrivate* pCxPrivate, bool pause)
+void SetPaused(ScriptInterface::CxPrivate* pCxPrivate, bool pause, bool sendMessage)
 {
 	if (!g_Game)
 	{
@@ -769,6 +778,9 @@ void SetPaused(ScriptInterface::CxPrivate* pCxPrivate, bool pause)
 	if (g_SoundManager)
 		g_SoundManager->Pause(pause);
 #endif
+
+	if (g_NetClient && sendMessage)
+		g_NetClient->SendPausedMessage(pause);
 }
 
 // Return the global frames-per-second value.
@@ -1021,19 +1033,18 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	scriptInterface.RegisterFunction<void, &StartNetworkGame>("StartNetworkGame");
 	scriptInterface.RegisterFunction<void, JS::HandleValue, int, &StartGame>("StartGame");
 	scriptInterface.RegisterFunction<void, &Script_EndGame>("EndGame");
-	scriptInterface.RegisterFunction<void, std::wstring, &StartNetworkHost>("StartNetworkHost");
-	scriptInterface.RegisterFunction<void, std::wstring, std::string, &StartNetworkJoin>("StartNetworkJoin");
+	scriptInterface.RegisterFunction<void, CStrW, u16, &StartNetworkHost>("StartNetworkHost");
+	scriptInterface.RegisterFunction<void, CStrW, CStr, u16, &StartNetworkJoin>("StartNetworkJoin");
+	scriptInterface.RegisterFunction<u16, &GetDefaultPort>("GetDefaultPort");
 	scriptInterface.RegisterFunction<void, &DisconnectNetworkGame>("DisconnectNetworkGame");
 	scriptInterface.RegisterFunction<std::string, &GetPlayerGUID>("GetPlayerGUID");
-	scriptInterface.RegisterFunction<bool, CStrW, bool, &KickPlayer>("KickPlayer");
+	scriptInterface.RegisterFunction<void, CStrW, bool, &KickPlayer>("KickPlayer");
 	scriptInterface.RegisterFunction<JS::Value, &PollNetworkClient>("PollNetworkClient");
 	scriptInterface.RegisterFunction<void, JS::HandleValue, &SetNetworkGameAttributes>("SetNetworkGameAttributes");
 	scriptInterface.RegisterFunction<void, int, std::string, &AssignNetworkPlayer>("AssignNetworkPlayer");
-	scriptInterface.RegisterFunction<void, std::string, int, &SetNetworkPlayerStatus>("SetNetworkPlayerStatus");
 	scriptInterface.RegisterFunction<void, &ClearAllPlayerReady>("ClearAllPlayerReady");
 	scriptInterface.RegisterFunction<void, std::wstring, &SendNetworkChat>("SendNetworkChat");
 	scriptInterface.RegisterFunction<void, int, &SendNetworkReady>("SendNetworkReady");
-	scriptInterface.RegisterFunction<void, &SendNetworkRejoined>("SendNetworkRejoined");
 	scriptInterface.RegisterFunction<JS::Value, &GetAIs>("GetAIs");
 	scriptInterface.RegisterFunction<JS::Value, &GetEngineInfo>("GetEngineInfo");
 
@@ -1049,6 +1060,7 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	// Misc functions
 	scriptInterface.RegisterFunction<std::wstring, std::wstring, &SetCursor>("SetCursor");
 	scriptInterface.RegisterFunction<bool, &IsVisualReplay>("IsVisualReplay");
+	scriptInterface.RegisterFunction<std::wstring, &GetCurrentReplayDirectory>("GetCurrentReplayDirectory");
 	scriptInterface.RegisterFunction<int, &GetPlayerID>("GetPlayerID");
 	scriptInterface.RegisterFunction<void, int, &SetPlayerID>("SetPlayerID");
 	scriptInterface.RegisterFunction<void, int, &SetViewedPlayer>("SetViewedPlayer");
@@ -1058,7 +1070,7 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	scriptInterface.RegisterFunction<bool, &AtlasIsAvailable>("AtlasIsAvailable");
 	scriptInterface.RegisterFunction<bool, &IsAtlasRunning>("IsAtlasRunning");
 	scriptInterface.RegisterFunction<JS::Value, VfsPath, &LoadMapSettings>("LoadMapSettings");
-	scriptInterface.RegisterFunction<JS::Value, &GetMapSettings>("GetMapSettings");
+	scriptInterface.RegisterFunction<JS::Value, &GetInitAttributes>("GetInitAttributes");
 	scriptInterface.RegisterFunction<float, &CameraGetX>("CameraGetX");
 	scriptInterface.RegisterFunction<float, &CameraGetZ>("CameraGetZ");
 	scriptInterface.RegisterFunction<void, entity_id_t, &CameraFollow>("CameraFollow");
@@ -1071,7 +1083,7 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	scriptInterface.RegisterFunction<JS::Value, &GetProfilerState>("GetProfilerState");
 	scriptInterface.RegisterFunction<void, &ExitProgram>("Exit");
 	scriptInterface.RegisterFunction<bool, &IsPaused>("IsPaused");
-	scriptInterface.RegisterFunction<void, bool, &SetPaused>("SetPaused");
+	scriptInterface.RegisterFunction<void, bool, bool, &SetPaused>("SetPaused");
 	scriptInterface.RegisterFunction<int, &GetFps>("GetFPS");
 	scriptInterface.RegisterFunction<std::wstring, int, &GetBuildTimestamp>("GetBuildTimestamp");
 	scriptInterface.RegisterFunction<JS::Value, std::wstring, &ReadJSONFile>("ReadJSONFile");
@@ -1101,4 +1113,5 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	scriptInterface.RegisterFunction<void, unsigned int, &EnableTimeWarpRecording>("EnableTimeWarpRecording");
 	scriptInterface.RegisterFunction<void, &RewindTimeWarp>("RewindTimeWarp");
 	scriptInterface.RegisterFunction<void, bool, &SetBoundingBoxDebugOverlay>("SetBoundingBoxDebugOverlay");
+	scriptInterface.RegisterFunction<CStrW, &GetSystemUsername>("GetSystemUsername");
 }
