@@ -32,6 +32,10 @@
 # include "SDL.h"
 # include "SDL_thread.h"
 
+#if !SDL_VERSION_ATLEAST(2,0,2)
+#error You are using an old libsdl release. At least libsdl2 >= 2.0.2 is required.
+#endif
+
 // if the compiler doesn't support inlining, this header will pull
 // in static bswap routines. doesn't matter - modern compilers
 // will strip them if unused, and this is more convenient than
@@ -39,13 +43,8 @@
 # include "SDL_endian.h"
 
 # if MSC_VERSION
-#  if SDL_VERSION_ATLEAST(2,0,0)
-#   pragma comment(lib, "SDL2")
-#   pragma comment(lib, "SDL2main")
-#  else
-#   pragma comment(lib, "SDL")
-#   pragma comment(lib, "SDLmain")
-#  endif
+#  pragma comment(lib, "SDL2")
+#  pragma comment(lib, "SDL2main")
 # endif
 
 // complete definition of our forward-declared SDL_Event (see sdl_fwd.h)
