@@ -73,6 +73,7 @@ that of Atlas depending on commandline parameters.
 #include "graphics/TextureManager.h"
 #include "gui/GUIManager.h"
 #include "renderer/Renderer.h"
+#include "scriptinterface/ScriptEngine.h"
 #include "simulation2/Simulation2.h"
 
 #if OS_UNIX
@@ -422,15 +423,6 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 		return;
 	}
 
-	// We need to initialise libxml2 in the main thread before
-	// any thread uses it. So initialise it here before we
-	// might run Atlas.
-	CXeromyces::Startup();
-
-	// Atlas handles the whole init/shutdown/etc sequence by itself;
-	if (ATLAS_RunIfOnCmdLine(args, false))
-		return;
-
 	const bool isReplay = args.Has("replay");
 	const bool isVisualReplay = args.Has("replay-visual");
 	const std::string replayFile = isReplay ? args.Get("replay") : (isVisualReplay ? args.Get("replay-visual") : "");
@@ -450,9 +442,27 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 		}
 	}
 
+	// We need to initialize SpiderMonkey and libxml2 in the main thread before
+	// any thread uses them. So initialize them here before we might run Atlas.
+	ScriptEngine scriptEngine;
+	CXeromyces::Startup();
+
+	if (ATLAS_RunIfOnCmdLine(args, false))
+	{
+		CXeromyces::Terminate();
+		return;
+	}
+
 	// run non-visual simulation replay if requested
 	if (isReplay)
 	{
+		if (!args.Has("mod"))
+		{
+			LOGERROR("At least one mod should be specified! Did you mean to add the argument '-mod=public'?");
+			CXeromyces::Terminate();
+			return;
+		}
+
 		Paths paths(args);
 		g_VFS = CreateVfs(20 * MiB);
 		g_VFS->Mount(L"cache/", paths.Cache(), VFS_MOUNT_ARCHIVABLE);
@@ -521,12 +531,8 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 	} while (restart);
 
 	if (restart_in_atlas)
-	{
 		ATLAS_RunIfOnCmdLine(args, true);
-		return;
-	}
 
-	// Shut down libxml2 (done here to match the Startup call)
 	CXeromyces::Terminate();
 }
 
