@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -15,19 +15,23 @@
  * along with 0 A.D.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-//Desc:  Contains classes for smooth splines
-//Borrowed from Game Programming Gems4.  (Slightly changed to better suit our purposes 
-//(and compatability).  Any references to external material can be found there
+/** 
+ * Contains classes for smooth splines
+ * Borrowed from Game Programming Gems 4. (Slightly changed to better suit our purposes 
+ * and compatability. Any references to external material can be found there.
+ */
 
 #ifndef INCLUDED_NUSPLINE
 #define INCLUDED_NUSPLINE
 
-#define MAX_SPLINE_NODES 40
-#include <stdlib.h>
+#define MAX_SPLINE_NODES 128
 
 #include "FixedVector3D.h"
 #include "Vector3D.h"
 
+/**
+ * Describes a node of the spline
+ */
 struct SplineData
 {
 	// Should be fixed, because used in the simulation
@@ -35,21 +39,26 @@ struct SplineData
 	CVector3D Velocity;
 	// TODO: make rotation as other spline
 	CFixedVector3D Rotation;
-	fixed Distance/*, DistanceOffset*/;	//DistanceOffset is to keep track of how far into the spline this node is
+	// Time interval to the previous node, should be 0 for the first node
+	fixed Distance;
 };
 
+
+/**
+ * Rounded Nonuniform Spline for describing spatial curves or paths with constant speed
+ */
 class RNSpline
 {
 public:
 
-	RNSpline() { NodeCount = 0; }
-	virtual ~RNSpline() {}
+	RNSpline();
+	virtual ~RNSpline();
 
 	void AddNode(const CFixedVector3D& pos);
 	void BuildSpline();
 	CVector3D GetPosition(float time) const;
 	CVector3D GetRotation(float time) const;
-	const std::vector<SplineData>& GetAllNodes() const { return Node; }
+	const std::vector<SplineData>& GetAllNodes() const;
 
 	fixed MaxDistance;
 	int NodeCount;
@@ -61,28 +70,37 @@ protected:
 	CVector3D GetEndVelocity(int index);
 };
 
+
+/**
+ * Smooth Nonuniform Spline for describing paths with smooth acceleration and deceleration,
+ * but without turning
+ */
 class SNSpline : public RNSpline
 {
 public:
-	virtual ~SNSpline() {}
-	void BuildSpline(){ RNSpline::BuildSpline(); Smooth(); Smooth(); Smooth(); }
+	virtual ~SNSpline();
+
+	void BuildSpline();
 	void Smooth();
 };
 
+
+/**
+ * Timed Nonuniform Spline for paths with different time intervals between nodes
+ */
 class TNSpline : public SNSpline
 {
 public:
-	virtual ~TNSpline() {}
+	virtual ~TNSpline();
 
 	void AddNode(const CFixedVector3D& pos, const CFixedVector3D& rotation, fixed timePeriod);
-	void PushNode() { Node.push_back( SplineData() ); }
 	void InsertNode(const int index, const CFixedVector3D& pos, const CFixedVector3D& rotation, fixed timePeriod);
 	void RemoveNode(const int index);
-	void UpdateNodeTime(const int index, fixed time);
 	void UpdateNodePos(const int index, const CFixedVector3D& pos);
+	void UpdateNodeTime(const int index, fixed time);
 
-	void BuildSpline(){ RNSpline::BuildSpline();  Smooth(); Smooth(); Smooth(); }
-	void Smooth(){ for (int x = 0; x < 3; ++x) { SNSpline::Smooth(); Constrain(); } }
+	void BuildSpline();
+	void Smooth();
 	void Constrain();
 };
 
