@@ -66,12 +66,14 @@ private:
 	CNetClient& m_Client;
 };
 
-CNetClient::CNetClient(CGame* game) :
+CNetClient::CNetClient(CGame* game, bool isLocalClient) :
 	m_Session(NULL),
 	m_UserName(L"anonymous"),
 	m_GUID(ps_generate_guid()), m_HostID((u32)-1), m_ClientTurnManager(NULL), m_Game(game),
 	m_GameAttributes(game->GetSimulation2()->GetScriptInterface().GetContext()),
-	m_LastConnectionCheck(0)
+	m_IsLocalClient(isLocalClient),
+	m_LastConnectionCheck(0),
+	m_Rejoin(false)
 {
 	m_Game->SetTurnManager(NULL); // delete the old local turn manager so we don't accidentally use it
 
@@ -103,6 +105,7 @@ CNetClient::CNetClient(CGame* game) :
 	AddTransition(NCS_JOIN_SYNCING, (uint)NMT_CHAT, NCS_JOIN_SYNCING, (void*)&OnChat, context);
 	AddTransition(NCS_JOIN_SYNCING, (uint)NMT_GAME_SETUP, NCS_JOIN_SYNCING, (void*)&OnGameSetup, context);
 	AddTransition(NCS_JOIN_SYNCING, (uint)NMT_PLAYER_ASSIGNMENT, NCS_JOIN_SYNCING, (void*)&OnPlayerAssignment, context);
+	AddTransition(NCS_JOIN_SYNCING, (uint)NMT_KICKED, NCS_JOIN_SYNCING, (void*)&OnKicked, context);
 	AddTransition(NCS_JOIN_SYNCING, (uint)NMT_CLIENT_TIMEOUT, NCS_JOIN_SYNCING, (void*)&OnClientTimeout, context);
 	AddTransition(NCS_JOIN_SYNCING, (uint)NMT_CLIENT_PERFORMANCE, NCS_JOIN_SYNCING, (void*)&OnClientPerformance, context);
 	AddTransition(NCS_JOIN_SYNCING, (uint)NMT_GAME_START, NCS_JOIN_SYNCING, (void*)&OnGameStart, context);
@@ -113,6 +116,7 @@ CNetClient::CNetClient(CGame* game) :
 	AddTransition(NCS_LOADING, (uint)NMT_CHAT, NCS_LOADING, (void*)&OnChat, context);
 	AddTransition(NCS_LOADING, (uint)NMT_GAME_SETUP, NCS_LOADING, (void*)&OnGameSetup, context);
 	AddTransition(NCS_LOADING, (uint)NMT_PLAYER_ASSIGNMENT, NCS_LOADING, (void*)&OnPlayerAssignment, context);
+	AddTransition(NCS_LOADING, (uint)NMT_KICKED, NCS_LOADING, (void*)&OnKicked, context);
 	AddTransition(NCS_LOADING, (uint)NMT_CLIENT_TIMEOUT, NCS_LOADING, (void*)&OnClientTimeout, context);
 	AddTransition(NCS_LOADING, (uint)NMT_CLIENT_PERFORMANCE, NCS_LOADING, (void*)&OnClientPerformance, context);
 	AddTransition(NCS_LOADING, (uint)NMT_LOADED_GAME, NCS_INGAME, (void*)&OnLoadedGame, context);
@@ -121,6 +125,7 @@ CNetClient::CNetClient(CGame* game) :
 	AddTransition(NCS_INGAME, (uint)NMT_KICKED, NCS_INGAME, (void*)&OnKicked, context);
 	AddTransition(NCS_INGAME, (uint)NMT_CLIENT_TIMEOUT, NCS_INGAME, (void*)&OnClientTimeout, context);
 	AddTransition(NCS_INGAME, (uint)NMT_CLIENT_PERFORMANCE, NCS_INGAME, (void*)&OnClientPerformance, context);
+	AddTransition(NCS_INGAME, (uint)NMT_CLIENT_PAUSED, NCS_INGAME, (void*)&OnClientPaused, context);
 	AddTransition(NCS_INGAME, (uint)NMT_CHAT, NCS_INGAME, (void*)&OnChat, context);
 	AddTransition(NCS_INGAME, (uint)NMT_GAME_SETUP, NCS_INGAME, (void*)&OnGameSetup, context);
 	AddTransition(NCS_INGAME, (uint)NMT_PLAYER_ASSIGNMENT, NCS_INGAME, (void*)&OnPlayerAssignment, context);
@@ -140,9 +145,8 @@ CNetClient::~CNetClient()
 
 void CNetClient::TraceMember(JSTracer *trc)
 {
-	std::deque<JS::Heap<JS::Value> >::iterator itr;
-	for (itr=m_GuiMessageQueue.begin(); itr != m_GuiMessageQueue.end(); ++itr)
-		JS_CallHeapValueTracer(trc, &*itr, "m_GuiMessageQueue");
+	for (JS::Heap<JS::Value>& guiMessage : m_GuiMessageQueue)
+		JS_CallValueTracer(trc, &guiMessage, "m_GuiMessageQueue");
 }
 
 void CNetClient::SetUserName(const CStrW& username)
@@ -152,10 +156,10 @@ void CNetClient::SetUserName(const CStrW& username)
 	m_UserName = username;
 }
 
-bool CNetClient::SetupConnection(const CStr& server)
+bool CNetClient::SetupConnection(const CStr& server, const u16 port)
 {
 	CNetClientSession* session = new CNetClientSession(*this);
-	bool ok = session->Connect(PS_DEFAULT_PORT, server);
+	bool ok = session->Connect(server, port, m_IsLocalClient);
 	SetAndOwnSession(session);
 	return ok;
 }
@@ -197,6 +201,7 @@ void CNetClient::CheckServerConnection()
 	m_LastConnectionCheck = now;
 
 	JSContext* cx = GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
 
 	// Report if we are losing the connection to the server
 	u32 lastReceived = m_Session->GetLastReceivedTime();
@@ -273,19 +278,19 @@ void CNetClient::PostPlayerAssignmentsToScript()
 	JSAutoRequest rq(cx);
 
 	JS::RootedValue msg(cx);
-	GetScriptInterface().Eval("({'type':'players', 'hosts':{}})", &msg);
+	GetScriptInterface().Eval("({'type':'players', 'newAssignments':{}})", &msg);
 
-	JS::RootedValue hosts(cx);
-	GetScriptInterface().GetProperty(msg, "hosts", &hosts);
+	JS::RootedValue newAssignments(cx);
+	GetScriptInterface().GetProperty(msg, "newAssignments", &newAssignments);
 
-	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
+	for (const std::pair<CStr, PlayerAssignment>& p : m_PlayerAssignments)
 	{
-		JS::RootedValue host(cx);
-		GetScriptInterface().Eval("({})", &host);
-		GetScriptInterface().SetProperty(host, "name", std::wstring(it->second.m_Name), false);
-		GetScriptInterface().SetProperty(host, "player", it->second.m_PlayerID, false);
-		GetScriptInterface().SetProperty(host, "status", it->second.m_Status, false);
-		GetScriptInterface().SetProperty(hosts, it->first.c_str(), host, false);
+		JS::RootedValue assignment(cx);
+		GetScriptInterface().Eval("({})", &assignment);
+		GetScriptInterface().SetProperty(assignment, "name", CStrW(p.second.m_Name), false);
+		GetScriptInterface().SetProperty(assignment, "player", p.second.m_PlayerID, false);
+		GetScriptInterface().SetProperty(assignment, "status", p.second.m_Status, false);
+		GetScriptInterface().SetProperty(newAssignments, p.first.c_str(), assignment, false);
 	}
 
 	PushGuiMessage(msg);
@@ -321,6 +326,21 @@ void CNetClient::HandleDisconnect(u32 reason)
 	SetCurrState(NCS_UNCONNECTED);
 }
 
+void CNetClient::SendGameSetupMessage(JS::MutableHandleValue attrs, ScriptInterface& scriptInterface)
+{
+	CGameSetupMessage gameSetup(scriptInterface);
+	gameSetup.m_Data = attrs;
+	SendMessage(&gameSetup);
+}
+
+void CNetClient::SendAssignPlayerMessage(const int playerID, const CStr& guid)
+{
+	CAssignPlayerMessage assignPlayer;
+	assignPlayer.m_PlayerID = playerID;
+	assignPlayer.m_GUID = guid;
+	SendMessage(&assignPlayer);
+}
+
 void CNetClient::SendChatMessage(const std::wstring& text)
 {
 	CChatMessage chat;
@@ -335,12 +355,39 @@ void CNetClient::SendReadyMessage(const int status)
 	SendMessage(&readyStatus);
 }
 
+void CNetClient::SendClearAllReadyMessage()
+{
+	CClearAllReadyMessage clearAllReady;
+	SendMessage(&clearAllReady);
+}
+
+void CNetClient::SendStartGameMessage()
+{
+	CGameStartMessage gameStart;
+	SendMessage(&gameStart);
+}
+
 void CNetClient::SendRejoinedMessage()
 {
 	CRejoinedMessage rejoinedMessage;
 	SendMessage(&rejoinedMessage);
 }
 
+void CNetClient::SendKickPlayerMessage(const CStrW& playerName, bool ban)
+{
+	CKickedMessage kickPlayer;
+	kickPlayer.m_Name = playerName;
+	kickPlayer.m_Ban = ban;
+	SendMessage(&kickPlayer);
+}
+
+void CNetClient::SendPausedMessage(bool pause)
+{
+	CClientPausedMessage pausedMessage;
+	pausedMessage.m_Pause = pause;
+	SendMessage(&pausedMessage);
+}
+
 bool CNetClient::HandleMessage(CNetMessage* message)
 {
 	// Handle non-FSM messages first
@@ -468,6 +515,7 @@ bool CNetClient::OnHandshakeResponse(void* context, CFsmEvent* event)
 	authenticate.m_GUID = client->m_GUID;
 	authenticate.m_Name = client->m_UserName;
 	authenticate.m_Password = L""; // TODO
+	authenticate.m_IsLocalClient = client->m_IsLocalClient;
 	client->SendMessage(&authenticate);
 
 	return true;
@@ -486,13 +534,12 @@ bool CNetClient::OnAuthenticate(void* context, CFsmEvent* event)
 
 	LOGMESSAGE("Net: Authentication result: host=%u, %s", message->m_HostID, utf8_from_wstring(message->m_Message));
 
-	bool  isRejoining = (message->m_Code == ARC_OK_REJOINING);
-
 	client->m_HostID = message->m_HostID;
+	client->m_Rejoin = message->m_Code == ARC_OK_REJOINING;
 
 	JS::RootedValue msg(cx);
 	client->GetScriptInterface().Eval("({'type':'netstatus','status':'authenticated'})", &msg);
-	client->GetScriptInterface().SetProperty(msg, "rejoining", isRejoining);
+	client->GetScriptInterface().SetProperty(msg, "rejoining", client->m_Rejoin);
 	client->PushGuiMessage(msg);
 
 	return true;
@@ -645,6 +692,7 @@ bool CNetClient::OnRejoined(void* context, CFsmEvent* event)
 
 	CNetClient* client = (CNetClient*)context;
 	JSContext* cx = client->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
 
 	CRejoinedMessage* message = (CRejoinedMessage*)event->GetParamRef();
 	JS::RootedValue msg(cx);
@@ -661,13 +709,15 @@ bool CNetClient::OnKicked(void *context, CFsmEvent* event)
 
 	CNetClient* client = (CNetClient*)context;
 	JSContext* cx = client->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
 
 	CKickedMessage* message = (CKickedMessage*)event->GetParamRef();
 	JS::RootedValue msg(cx);
 
 	client->GetScriptInterface().Eval("({})", &msg);
 	client->GetScriptInterface().SetProperty(msg, "username", message->m_Name);
-	client->GetScriptInterface().SetProperty(msg, "type", message->m_Ban ? std::string("banned") : std::string("kicked"));
+	client->GetScriptInterface().SetProperty(msg, "type", CStr("kicked"));
+	client->GetScriptInterface().SetProperty(msg, "banned", message->m_Ban != 0);
 	client->PushGuiMessage(msg);
 
 	return true;
@@ -681,6 +731,7 @@ bool CNetClient::OnClientTimeout(void *context, CFsmEvent* event)
 
 	CNetClient* client = (CNetClient*)context;
 	JSContext* cx = client->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
 
 	if (client->GetCurrState() == NCS_LOADING)
 		return true;
@@ -704,29 +755,48 @@ bool CNetClient::OnClientPerformance(void *context, CFsmEvent* event)
 
 	CNetClient* client = (CNetClient*)context;
 	JSContext* cx = client->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
 
 	if (client->GetCurrState() == NCS_LOADING)
 		return true;
 
 	CClientPerformanceMessage* message = (CClientPerformanceMessage*)event->GetParamRef();
-	std::vector<CClientPerformanceMessage::S_m_Clients> &clients = message->m_Clients;
 
 	// Display warnings for other clients with bad ping
-	for (size_t i = 0; i < clients.size(); ++i)
+	for (size_t i = 0; i < message->m_Clients.size(); ++i)
 	{
-		if (clients[i].m_MeanRTT < DEFAULT_TURN_LENGTH_MP || clients[i].m_GUID == client->m_GUID)
+		if (message->m_Clients[i].m_MeanRTT < DEFAULT_TURN_LENGTH_MP || message->m_Clients[i].m_GUID == client->m_GUID)
 			continue;
 
 		JS::RootedValue msg(cx);
 		client->GetScriptInterface().Eval("({ 'type':'netwarn', 'warntype': 'client-latency' })", &msg);
-		client->GetScriptInterface().SetProperty(msg, "guid", clients[i].m_GUID);
-		client->GetScriptInterface().SetProperty(msg, "meanRTT", clients[i].m_MeanRTT);
+		client->GetScriptInterface().SetProperty(msg, "guid", message->m_Clients[i].m_GUID);
+		client->GetScriptInterface().SetProperty(msg, "meanRTT", message->m_Clients[i].m_MeanRTT);
 		client->PushGuiMessage(msg);
 	}
 
 	return true;
 }
 
+bool CNetClient::OnClientPaused(void *context, CFsmEvent *event)
+{
+	ENSURE(event->GetType() == (uint)NMT_CLIENT_PAUSED);
+
+	CNetClient* client = (CNetClient*)context;
+	JSContext* cx = client->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+
+	CClientPausedMessage* message = (CClientPausedMessage*)event->GetParamRef();
+
+	JS::RootedValue msg(cx);
+	client->GetScriptInterface().Eval("({ 'type':'paused' })", &msg);
+	client->GetScriptInterface().SetProperty(msg, "pause", message->m_Pause != 0);
+	client->GetScriptInterface().SetProperty(msg, "guid", message->m_GUID);
+	client->PushGuiMessage(msg);
+
+	return true;
+}
+
 bool CNetClient::OnLoadedGame(void* context, CFsmEvent* event)
 {
 	ENSURE(event->GetType() == (uint)NMT_LOADED_GAME);
@@ -743,6 +813,10 @@ bool CNetClient::OnLoadedGame(void* context, CFsmEvent* event)
 	client->GetScriptInterface().Eval("({'type':'netstatus','status':'active'})", &msg);
 	client->PushGuiMessage(msg);
 
+	// If we have rejoined an in progress game, send the rejoined message to the server.
+	if (client->m_Rejoin)
+		client->SendRejoinedMessage();
+
 	return true;
 }
 
