@@ -28,7 +28,7 @@
 
 #define PS_PROTOCOL_MAGIC				0x5073013f		// 'P', 's', 0x01, '?'
 #define PS_PROTOCOL_MAGIC_RESPONSE		0x50630121		// 'P', 'c', 0x01, '!'
-#define PS_PROTOCOL_VERSION				0x01010011		// Arbitrary protocol
+#define PS_PROTOCOL_VERSION				0x01010014		// Arbitrary protocol
 #define PS_DEFAULT_PORT					0x5073			// 'P', 's'
 
 // Defines the list of message types. The order of the list must not change.
@@ -37,17 +37,22 @@
 // positive values.
 enum NetMessageType
 {
-	NMT_CONNECT_COMPLETE = -256,	// Connection is complete
+	NMT_CONNECT_COMPLETE = -256,
 	NMT_CONNECTION_LOST,
-	NMT_INVALID = 0,		// Invalid message
-	NMT_SERVER_HANDSHAKE,	// Handshake stage
+	NMT_INVALID = 0,
+
+	NMT_SERVER_HANDSHAKE,
 	NMT_CLIENT_HANDSHAKE,
 	NMT_SERVER_HANDSHAKE_RESPONSE,
-	NMT_AUTHENTICATE,		// Authentication stage
+
+	NMT_AUTHENTICATE,
 	NMT_AUTHENTICATE_RESULT,
-	NMT_CHAT,		// Common chat message
+
+	NMT_CHAT,
 	NMT_READY,
+	NMT_CLEAR_ALL_READY,
 	NMT_GAME_SETUP,
+	NMT_ASSIGN_PLAYER,
 	NMT_PLAYER_ASSIGNMENT,
 
 	NMT_FILE_TRANSFER_REQUEST,
@@ -62,14 +67,16 @@ enum NetMessageType
 
 	NMT_CLIENT_TIMEOUT,
 	NMT_CLIENT_PERFORMANCE,
+	NMT_CLIENT_PAUSED,
 
 	NMT_LOADED_GAME,
 	NMT_GAME_START,
 	NMT_END_COMMAND_BATCH,
+
 	NMT_SYNC_CHECK,	// OOS-detection hash checking
 	NMT_SYNC_ERROR,	// OOS-detection error
-	NMT_SIMULATION_COMMAND,
-	NMT_LAST				// Last message in the list
+
+	NMT_SIMULATION_COMMAND
 };
 
 // Authentication result codes
@@ -109,9 +116,10 @@ START_NMT_CLASS_(SrvHandshakeResponse, NMT_SERVER_HANDSHAKE_RESPONSE)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(Authenticate, NMT_AUTHENTICATE)
-	NMT_FIELD(CStr8, m_GUID)
+	NMT_FIELD(CStr, m_GUID)
 	NMT_FIELD(CStrW, m_Name)
 	NMT_FIELD(CStrW, m_Password)
+	NMT_FIELD_INT(m_IsLocalClient, u8, 1)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(AuthenticateResult, NMT_AUTHENTICATE_RESULT)
@@ -121,18 +129,21 @@ START_NMT_CLASS_(AuthenticateResult, NMT_AUTHENTICATE_RESULT)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(Chat, NMT_CHAT)
-	NMT_FIELD(CStr8, m_GUID) // ignored when client->server, valid when server->client
+	NMT_FIELD(CStr, m_GUID) // ignored when client->server, valid when server->client
 	NMT_FIELD(CStrW, m_Message)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(Ready, NMT_READY)
-	NMT_FIELD(CStr8, m_GUID)
+	NMT_FIELD(CStr, m_GUID)
 	NMT_FIELD_INT(m_Status, u8, 1)
 END_NMT_CLASS()
 
+START_NMT_CLASS_(ClearAllReady, NMT_CLEAR_ALL_READY)
+END_NMT_CLASS()
+
 START_NMT_CLASS_(PlayerAssignment, NMT_PLAYER_ASSIGNMENT)
 	NMT_START_ARRAY(m_Hosts)
-		NMT_FIELD(CStr8, m_GUID)
+		NMT_FIELD(CStr, m_GUID)
 		NMT_FIELD(CStrW, m_Name)
 		NMT_FIELD_INT(m_PlayerID, i8, 1)
 		NMT_FIELD_INT(m_Status, u8, 1)
@@ -150,7 +161,7 @@ END_NMT_CLASS()
 
 START_NMT_CLASS_(FileTransferData, NMT_FILE_TRANSFER_DATA)
 	NMT_FIELD_INT(m_RequestID, u32, 4)
-	NMT_FIELD(CStr8, m_Data)
+	NMT_FIELD(CStr, m_Data)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(FileTransferAck, NMT_FILE_TRANSFER_ACK)
@@ -162,7 +173,7 @@ START_NMT_CLASS_(JoinSyncStart, NMT_JOIN_SYNC_START)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(Rejoined, NMT_REJOINED)
-	NMT_FIELD(CStr8, m_GUID)
+	NMT_FIELD(CStr, m_GUID)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(Kicked, NMT_KICKED)
@@ -171,17 +182,22 @@ START_NMT_CLASS_(Kicked, NMT_KICKED)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(ClientTimeout, NMT_CLIENT_TIMEOUT)
-	NMT_FIELD(CStr8, m_GUID)
+	NMT_FIELD(CStr, m_GUID)
 	NMT_FIELD_INT(m_LastReceivedTime, u32, 4)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(ClientPerformance, NMT_CLIENT_PERFORMANCE)
 	NMT_START_ARRAY(m_Clients)
-		NMT_FIELD(CStr8, m_GUID)
+		NMT_FIELD(CStr, m_GUID)
 		NMT_FIELD_INT(m_MeanRTT, u32, 4)
 	NMT_END_ARRAY()
 END_NMT_CLASS()
 
+START_NMT_CLASS_(ClientPaused, NMT_CLIENT_PAUSED)
+	NMT_FIELD(CStr, m_GUID)
+	NMT_FIELD_INT(m_Pause, u8, 1)
+END_NMT_CLASS()
+
 START_NMT_CLASS_(LoadedGame, NMT_LOADED_GAME)
 	NMT_FIELD_INT(m_CurrentTurn, u32, 4)
 END_NMT_CLASS()
@@ -207,6 +223,11 @@ START_NMT_CLASS_(SyncError, NMT_SYNC_ERROR)
 	NMT_END_ARRAY()
 END_NMT_CLASS()
 
+START_NMT_CLASS_(AssignPlayer, NMT_ASSIGN_PLAYER)
+	NMT_FIELD_INT(m_PlayerID, i8, 1)
+	NMT_FIELD(CStr, m_GUID)
+END_NMT_CLASS()
+
 END_NMTS()
 
 #else
