@@ -28,6 +28,7 @@
 #include "lib/external_libraries/enet.h"
 #include "ps/CLogger.h"
 #include "ps/ConfigDB.h"
+#include "ps/Profile.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "scriptinterface/ScriptRuntime.h"
 #include "simulation2/Simulation2.h"
@@ -43,9 +44,9 @@
  * Number of peers to allocate for the enet host.
  * Limited by ENET_PROTOCOL_MAXIMUM_PEER_ID (4096).
  *
- * At most 8 players, 16 observers and 1 temporary connection to send the "server full" disconnect-reason.
+ * At most 8 players, 32 observers and 1 temporary connection to send the "server full" disconnect-reason.
  */
-#define MAX_CLIENTS 25
+#define MAX_CLIENTS 41
 
 #define	DEFAULT_SERVER_NAME			L"Unnamed Server"
 #define DEFAULT_WELCOME_MESSAGE		L"Welcome"
@@ -90,11 +91,11 @@ public:
 
 		// Find the session corresponding to the rejoining host (if any)
 		CNetServerSession* session = NULL;
-		for (size_t i = 0; i < m_Server.m_Sessions.size(); ++i)
+		for (CNetServerSession* serverSession : m_Server.m_Sessions)
 		{
-			if (m_Server.m_Sessions[i]->GetHostID() == m_RejoinerHostID)
+			if (serverSession->GetHostID() == m_RejoinerHostID)
 			{
-				session = m_Server.m_Sessions[i];
+				session = serverSession;
 				break;
 			}
 		}
@@ -157,21 +158,19 @@ CNetServerWorker::~CNetServerWorker()
 
 	delete m_Stats;
 
-	for (size_t i = 0; i < m_Sessions.size(); ++i)
+	for (CNetServerSession* session : m_Sessions)
 	{
-		m_Sessions[i]->DisconnectNow(NDR_SERVER_SHUTDOWN);
-		delete m_Sessions[i];
+		session->DisconnectNow(NDR_SERVER_SHUTDOWN);
+		delete session;
 	}
 
 	if (m_Host)
-	{
 		enet_host_destroy(m_Host);
-	}
 
 	delete m_ServerTurnManager;
 }
 
-bool CNetServerWorker::SetupConnection()
+bool CNetServerWorker::SetupConnection(const u16 port)
 {
 	ENSURE(m_State == SERVER_STATE_UNCONNECTED);
 	ENSURE(!m_Host);
@@ -179,7 +178,7 @@ bool CNetServerWorker::SetupConnection()
 	// Bind to default host
 	ENetAddress addr;
 	addr.host = ENET_HOST_ANY;
-	addr.port = PS_DEFAULT_PORT;
+	addr.port = port;
 
 	// Create ENet server
 	m_Host = enet_host_create(&addr, MAX_CLIENTS, CHANNEL_COUNT, 0, 0);
@@ -346,18 +345,13 @@ bool CNetServerWorker::Broadcast(const CNetMessage* message)
 	bool ok = true;
 
 	// Send to all sessions that are active and has finished authentication
-	for (size_t i = 0; i < m_Sessions.size(); ++i)
-	{
-		if (m_Sessions[i]->GetCurrState() == NSS_PREGAME || m_Sessions[i]->GetCurrState() == NSS_INGAME)
-		{
-			if (!m_Sessions[i]->SendMessage(message))
+	// TODO: this does lots of repeated message serialisation if we have lots
+	// of remote peers; could do it more efficiently if that's a real problem
+	for (CNetServerSession* session : m_Sessions)
+		if (session->GetCurrState() == NSS_PREGAME || session->GetCurrState() == NSS_INGAME)
+			if (!session->SendMessage(message))
 				ok = false;
 
-			// TODO: this does lots of repeated message serialisation if we have lots
-			// of remote peers; could do it more efficiently if that's a real problem
-		}
-	}
-
 	return ok;
 }
 
@@ -378,7 +372,7 @@ void CNetServerWorker::Run()
 	// To avoid the need for JS_SetContextThread, we create and use and destroy
 	// the script interface entirely within this network thread
 	m_ScriptInterface = new ScriptInterface("Engine", "Net server", ScriptInterface::CreateRuntime(g_ScriptRuntime));
-	m_GameAttributes.set(m_ScriptInterface->GetJSRuntime(), JS::UndefinedValue());
+	m_GameAttributes.init(m_ScriptInterface->GetJSRuntime(), JS::UndefinedValue());
 
 	while (true)
 	{
@@ -394,7 +388,6 @@ void CNetServerWorker::Run()
 	}
 
 	// Clear roots before deleting their context
-	m_GameAttributes.clear();
 	m_SavedCommands.clear();
 
 	SAFE_DELETE(m_ScriptInterface);
@@ -411,10 +404,7 @@ bool CNetServerWorker::RunStep()
 	JSContext* cx = m_ScriptInterface->GetContext();
 	JSAutoRequest rq(cx);
 
-	std::vector<std::pair<int, CStr> > newAssignPlayer;
 	std::vector<bool> newStartGame;
-	std::vector<std::pair<CStr, int> > newPlayerReady;
-	std::vector<bool> newPlayerResetReady;
 	std::vector<std::string> newGameAttributes;
 	std::vector<u32> newTurnLength;
 
@@ -425,22 +415,10 @@ bool CNetServerWorker::RunStep()
 			return false;
 
 		newStartGame.swap(m_StartGameQueue);
-		newPlayerReady.swap(m_PlayerReadyQueue);
-		newPlayerResetReady.swap(m_PlayerResetReadyQueue);
-		newAssignPlayer.swap(m_AssignPlayerQueue);
 		newGameAttributes.swap(m_GameAttributesQueue);
 		newTurnLength.swap(m_TurnLengthQueue);
 	}
 
-	for (size_t i = 0; i < newAssignPlayer.size(); ++i)
-		AssignPlayer(newAssignPlayer[i].first, newAssignPlayer[i].second);
-
-	for (size_t i = 0; i < newPlayerReady.size(); ++i)
-		SetPlayerReady(newPlayerReady[i].first, newPlayerReady[i].second);
-
-	if (!newPlayerResetReady.empty())
-		ClearAllPlayerReady();
-
 	if (!newGameAttributes.empty())
 	{
 		JS::RootedValue gameAttributesVal(cx);
@@ -456,8 +434,8 @@ bool CNetServerWorker::RunStep()
 		StartGame();
 
 	// Perform file transfers
-	for (size_t i = 0; i < m_Sessions.size(); ++i)
-		m_Sessions[i]->GetFileTransferer().Poll();
+	for (CNetServerSession* session : m_Sessions)
+		session->GetFileTransferer().Poll();
 
 	CheckClientConnections();
 
@@ -493,12 +471,6 @@ bool CNetServerWorker::RunStep()
 
 		CNetServerSession* session = new CNetServerSession(*this, event.peer);
 
-		// Prevent the local client of the host from timing out too quickly
-#if (ENET_VERSION >= ENET_VERSION_CREATE(1, 3, 4))
-		if (session->GetIPAddress() == "127.0.0.1")
-			enet_peer_timeout(event.peer, 0, MAXIMUM_HOST_TIMEOUT, MAXIMUM_HOST_TIMEOUT);
-#endif
-
 		m_Sessions.push_back(session);
 
 		SetupSession(session);
@@ -530,6 +502,9 @@ bool CNetServerWorker::RunStep()
 			event.peer->data = NULL;
 		}
 
+		if (m_State == SERVER_STATE_LOADING)
+			CheckGameLoadStatus(NULL);
+
 		break;
 	}
 
@@ -607,8 +582,14 @@ void CNetServerWorker::CheckClientConnections()
 		// (since that will show the locally triggered warning instead)
 		if (message)
 			for (size_t j = 0; j < m_Sessions.size(); ++j)
-				if (i != j)
+			{
+				if (i != j && (
+				    m_Sessions[j]->GetCurrState() == NSS_PREGAME ||
+				    m_Sessions[j]->GetCurrState() == NSS_INGAME))
+				{
 					m_Sessions[j]->SendMessage(message);
+				}
+			}
 
 		SAFE_DELETE(message);
 	}
@@ -635,8 +616,7 @@ void CNetServerWorker::HandleMessageReceive(const CNetMessage* message, CNetServ
 	}
 
 	// Update FSM
-	bool ok = session->Update(message->GetType(), (void*)message);
-	if (!ok)
+	if (!session->Update(message->GetType(), (void*)message))
 		LOGERROR("Net server: Error running FSM update (type=%d state=%d)", (int)message->GetType(), (int)session->GetCurrState());
 }
 
@@ -657,12 +637,20 @@ void CNetServerWorker::SetupSession(CNetServerSession* session)
 	session->AddTransition(NSS_PREGAME, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
 	session->AddTransition(NSS_PREGAME, (uint)NMT_CHAT, NSS_PREGAME, (void*)&OnChat, context);
 	session->AddTransition(NSS_PREGAME, (uint)NMT_READY, NSS_PREGAME, (void*)&OnReady, context);
+	session->AddTransition(NSS_PREGAME, (uint)NMT_CLEAR_ALL_READY, NSS_PREGAME, (void*)&OnClearAllReady, context);
+	session->AddTransition(NSS_PREGAME, (uint)NMT_GAME_SETUP, NSS_PREGAME, (void*)&OnGameSetup, context);
+	session->AddTransition(NSS_PREGAME, (uint)NMT_ASSIGN_PLAYER, NSS_PREGAME, (void*)&OnAssignPlayer, context);
+	session->AddTransition(NSS_PREGAME, (uint)NMT_KICKED, NSS_PREGAME, (void*)&OnKickPlayer, context);
+	session->AddTransition(NSS_PREGAME, (uint)NMT_GAME_START, NSS_PREGAME, (void*)&OnStartGame, context);
 	session->AddTransition(NSS_PREGAME, (uint)NMT_LOADED_GAME, NSS_INGAME, (void*)&OnLoadedGame, context);
 
+	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_KICKED, NSS_JOIN_SYNCING, (void*)&OnKickPlayer, context);
 	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
 	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_LOADED_GAME, NSS_INGAME, (void*)&OnJoinSyncingLoadedGame, context);
 
 	session->AddTransition(NSS_INGAME, (uint)NMT_REJOINED, NSS_INGAME, (void*)&OnRejoined, context);
+	session->AddTransition(NSS_INGAME, (uint)NMT_KICKED, NSS_INGAME, (void*)&OnKickPlayer, context);
+	session->AddTransition(NSS_INGAME, (uint)NMT_CLIENT_PAUSED, NSS_INGAME, (void*)&OnClientPaused, context);
 	session->AddTransition(NSS_INGAME, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
 	session->AddTransition(NSS_INGAME, (uint)NMT_CHAT, NSS_INGAME, (void*)&OnChat, context);
 	session->AddTransition(NSS_INGAME, (uint)NMT_SIMULATION_COMMAND, NSS_INGAME, (void*)&OnInGame, context);
@@ -692,11 +680,11 @@ void CNetServerWorker::OnUserJoin(CNetServerSession* session)
 {
 	AddPlayer(session->GetGUID(), session->GetUserName());
 
-	if (m_HostGUID.empty() && session->GetIPAddress() == "127.0.0.1")
+	if (m_HostGUID.empty() && session->IsLocalClient())
 		m_HostGUID = session->GetGUID();
 
 	CGameSetupMessage gameSetupMessage(GetScriptInterface());
-	gameSetupMessage.m_Data = m_GameAttributes.get();
+	gameSetupMessage.m_Data = m_GameAttributes;
 	session->SendMessage(&gameSetupMessage);
 
 	CPlayerAssignmentMessage assignMessage;
@@ -706,6 +694,10 @@ void CNetServerWorker::OnUserJoin(CNetServerSession* session)
 
 void CNetServerWorker::OnUserLeave(CNetServerSession* session)
 {
+	std::vector<CStr>::iterator pausing = std::find(m_PausingPlayers.begin(), m_PausingPlayers.end(), session->GetGUID());
+	if (pausing != m_PausingPlayers.end())
+		m_PausingPlayers.erase(pausing);
+
 	RemovePlayer(session->GetGUID());
 
 	if (m_ServerTurnManager && session->GetCurrState() != NSS_JOIN_SYNCING)
@@ -719,9 +711,9 @@ void CNetServerWorker::AddPlayer(const CStr& guid, const CStrW& name)
 {
 	// Find all player IDs in active use; we mustn't give them to a second player (excluding the unassigned ID: -1)
 	std::set<i32> usedIDs;
-	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
-		if (it->second.m_Enabled && it->second.m_PlayerID != -1)
-			usedIDs.insert(it->second.m_PlayerID);
+	for (const std::pair<CStr, PlayerAssignment>& p : m_PlayerAssignments)
+		if (p.second.m_Enabled && p.second.m_PlayerID != -1)
+			usedIDs.insert(p.second.m_PlayerID);
 
 	// If the player is rejoining after disconnecting, try to give them
 	// back their old player ID
@@ -787,7 +779,7 @@ void CNetServerWorker::ClearAllPlayerReady()
 	SendPlayerAssignments();
 }
 
-bool CNetServerWorker::KickPlayer(const CStrW& playerName, const bool ban)
+void CNetServerWorker::KickPlayer(const CStrW& playerName, const bool ban)
 {
 	// Find the user with that name
 	std::vector<CNetServerSession*>::iterator it = std::find_if(m_Sessions.begin(), m_Sessions.end(),
@@ -795,7 +787,7 @@ bool CNetServerWorker::KickPlayer(const CStrW& playerName, const bool ban)
 
 	// and return if no one or the host has that name
 	if (it == m_Sessions.end() || (*it)->GetGUID() == m_HostGUID)
-		return false;
+		return;
 
 	if (ban)
 	{
@@ -804,8 +796,8 @@ bool CNetServerWorker::KickPlayer(const CStrW& playerName, const bool ban)
 			m_BannedPlayers.push_back(playerName);
 
 		// Remember IP address
-		CStr ipAddress = (*it)->GetIPAddress();
-		if (!ipAddress.empty() && std::find(m_BannedIPs.begin(), m_BannedIPs.end(), ipAddress) == m_BannedIPs.end())
+		u32 ipAddress = (*it)->GetIPAddress();
+		if (std::find(m_BannedIPs.begin(), m_BannedIPs.end(), ipAddress) == m_BannedIPs.end())
 			m_BannedIPs.push_back(ipAddress);
 	}
 
@@ -816,18 +808,18 @@ bool CNetServerWorker::KickPlayer(const CStrW& playerName, const bool ban)
 	CKickedMessage kickedMessage;
 	kickedMessage.m_Name = playerName;
 	kickedMessage.m_Ban = ban;
-	Broadcast(&kickedMessage);
-
-	return true;
+	for (CNetServerSession* session : m_Sessions)
+		if (session->GetCurrState() > NSS_AUTHENTICATE)
+			session->SendMessage(&kickedMessage);
 }
 
 void CNetServerWorker::AssignPlayer(int playerID, const CStr& guid)
 {
 	// Remove anyone who's already assigned to this player
-	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
+	for (std::pair<const CStr, PlayerAssignment>& p : m_PlayerAssignments)
 	{
-		if (it->second.m_PlayerID == playerID)
-			it->second.m_PlayerID = -1;
+		if (p.second.m_PlayerID == playerID)
+			p.second.m_PlayerID = -1;
 	}
 
 	// Update this host's assignment if it exists
@@ -839,16 +831,16 @@ void CNetServerWorker::AssignPlayer(int playerID, const CStr& guid)
 
 void CNetServerWorker::ConstructPlayerAssignmentMessage(CPlayerAssignmentMessage& message)
 {
-	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
+	for (const std::pair<CStr, PlayerAssignment>& p : m_PlayerAssignments)
 	{
-		if (!it->second.m_Enabled)
+		if (!p.second.m_Enabled)
 			continue;
 
 		CPlayerAssignmentMessage::S_m_Hosts h;
-		h.m_GUID = it->first;
-		h.m_Name = it->second.m_Name;
-		h.m_PlayerID = it->second.m_PlayerID;
-		h.m_Status = it->second.m_Status;
+		h.m_GUID = p.first;
+		h.m_Name = p.second.m_Name;
+		h.m_PlayerID = p.second.m_PlayerID;
+		h.m_Status = p.second.m_Status;
 		message.m_Hosts.push_back(h);
 	}
 }
@@ -955,18 +947,20 @@ bool CNetServerWorker::OnAuthenticate(void* context, CFsmEvent* event)
 		int disconnectedPlayers = 0;
 		int connectedPlayers = 0;
 		// (TODO: if GUIDs were stable, we should use them instead)
-		for (PlayerAssignmentMap::iterator it = server.m_PlayerAssignments.begin(); it != server.m_PlayerAssignments.end(); ++it)
+		for (const std::pair<CStr, PlayerAssignment>& p : server.m_PlayerAssignments)
 		{
-			if (!it->second.m_Enabled && it->second.m_Name == username)
+			const PlayerAssignment& assignment = p.second;
+
+			if (!assignment.m_Enabled && assignment.m_Name == username)
 			{
-				isObserver = it->second.m_PlayerID == -1;
+				isObserver = assignment.m_PlayerID == -1;
 				isRejoining = true;
 			}
 
-			if (it->second.m_PlayerID == -1)
+			if (assignment.m_PlayerID == -1)
 				continue;
 
-			if (it->second.m_Enabled)
+			if (assignment.m_Enabled)
 				++connectedPlayers;
 			else
 				++disconnectedPlayers;
@@ -999,6 +993,7 @@ bool CNetServerWorker::OnAuthenticate(void* context, CFsmEvent* event)
 	session->SetUserName(username);
 	session->SetGUID(message->m_GUID);
 	session->SetHostID(newHostID);
+	session->SetLocalClient(message->m_IsLocalClient);
 
 	CAuthenticateResultMessage authenticateResult;
 	authenticateResult.m_Code = isRejoining ? ARC_OK_REJOINING : ARC_OK;
@@ -1045,7 +1040,7 @@ bool CNetServerWorker::OnInGame(void* context, CFsmEvent* event)
 		JSContext* cx = scriptInterface.GetContext();
 		JSAutoRequest rq(cx);
 		JS::RootedValue settings(cx);
-		scriptInterface.GetProperty(server.m_GameAttributes.get(), "settings", &settings);
+		scriptInterface.GetProperty(server.m_GameAttributes, "settings", &settings);
 		if (scriptInterface.HasProperty(settings, "CheatsEnabled"))
 			scriptInterface.GetProperty(settings, "CheatsEnabled", cheatsEnabled);
 
@@ -1072,6 +1067,7 @@ bool CNetServerWorker::OnInGame(void* context, CFsmEvent* event)
 	}
 	else if (message->GetType() == (uint)NMT_END_COMMAND_BATCH)
 	{
+		// The turn-length field is ignored
 		CEndCommandBatchMessage* endMessage = static_cast<CEndCommandBatchMessage*> (message);
 		server.m_ServerTurnManager->NotifyFinishedClientCommands(session->GetHostID(), endMessage->m_Turn);
 	}
@@ -1107,6 +1103,61 @@ bool CNetServerWorker::OnReady(void* context, CFsmEvent* event)
 	message->m_GUID = session->GetGUID();
 
 	server.Broadcast(message);
+	server.SetPlayerReady(message->m_GUID, message->m_Status);
+
+	return true;
+}
+
+bool CNetServerWorker::OnClearAllReady(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_CLEAR_ALL_READY);
+
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	if (session->GetGUID() == server.m_HostGUID)
+		server.ClearAllPlayerReady();
+
+	return true;
+}
+
+bool CNetServerWorker::OnGameSetup(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_GAME_SETUP);
+
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	if (session->GetGUID() == server.m_HostGUID)
+	{
+		CGameSetupMessage* message = (CGameSetupMessage*)event->GetParamRef();
+		server.UpdateGameAttributes(&(message->m_Data));
+	}
+	return true;
+}
+
+bool CNetServerWorker::OnAssignPlayer(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_ASSIGN_PLAYER);
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	if (session->GetGUID() == server.m_HostGUID)
+	{
+		CAssignPlayerMessage* message = (CAssignPlayerMessage*)event->GetParamRef();
+		server.AssignPlayer(message->m_PlayerID, message->m_GUID);
+	}
+	return true;
+}
+
+bool CNetServerWorker::OnStartGame(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_GAME_START);
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	if (session->GetGUID() == server.m_HostGUID)
+		server.StartGame();
 
 	return true;
 }
@@ -1191,6 +1242,30 @@ bool CNetServerWorker::OnRejoined(void* context, CFsmEvent* event)
 
 	server.Broadcast(message);
 
+	// Send all pausing players to the rejoined client.
+	for (const CStr& guid : server.m_PausingPlayers)
+	{
+		CClientPausedMessage pausedMessage;
+		pausedMessage.m_GUID = guid;
+		pausedMessage.m_Pause = true;
+		session->SendMessage(&pausedMessage);
+	}
+
+	return true;
+}
+
+bool CNetServerWorker::OnKickPlayer(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_KICKED);
+
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	if (session->GetGUID() == server.m_HostGUID)
+	{
+		CKickedMessage* message = (CKickedMessage*)event->GetParamRef();
+		server.KickPlayer(message->m_Name, message->m_Ban);
+	}
 	return true;
 }
 
@@ -1206,13 +1281,50 @@ bool CNetServerWorker::OnDisconnect(void* context, CFsmEvent* event)
 	return true;
 }
 
+bool CNetServerWorker::OnClientPaused(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_CLIENT_PAUSED);
+
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	CClientPausedMessage* message = (CClientPausedMessage*)event->GetParamRef();
+
+	message->m_GUID = session->GetGUID();
+
+	// Update the list of pausing players.
+	std::vector<CStr>::iterator player = std::find(server.m_PausingPlayers.begin(), server.m_PausingPlayers.end(), session->GetGUID());
+
+	if (message->m_Pause)
+	{
+		if (player != server.m_PausingPlayers.end())
+			return true;
+
+		server.m_PausingPlayers.push_back(session->GetGUID());
+	}
+	else
+	{
+		if (player == server.m_PausingPlayers.end())
+			return true;
+
+		server.m_PausingPlayers.erase(player);
+	}
+
+	// Send messages to clients that are in game, and are not the client who paused.
+	for (CNetServerSession* session : server.m_Sessions)
+	{
+		if (session->GetCurrState() == NSS_INGAME && message->m_GUID != session->GetGUID())
+			session->SendMessage(message);
+	}
+
+	return true;
+}
+
 void CNetServerWorker::CheckGameLoadStatus(CNetServerSession* changedSession)
 {
-	for (size_t i = 0; i < m_Sessions.size(); ++i)
-	{
-		if (m_Sessions[i] != changedSession && m_Sessions[i]->GetCurrState() != NSS_INGAME)
+	for (const CNetServerSession* session : m_Sessions)
+		if (session != changedSession && session->GetCurrState() != NSS_INGAME)
 			return;
-	}
 
 	CLoadedGameMessage loaded;
 	loaded.m_CurrentTurn = 0;
@@ -1225,13 +1337,13 @@ void CNetServerWorker::StartGame()
 {
 	m_ServerTurnManager = new CNetServerTurnManager(*this);
 
-	for (size_t i = 0; i < m_Sessions.size(); ++i)
-		m_ServerTurnManager->InitialiseClient(m_Sessions[i]->GetHostID(), 0); // TODO: only for non-observers
+	for (const CNetServerSession* session : m_Sessions)
+		m_ServerTurnManager->InitialiseClient(session->GetHostID(), 0); // TODO: only for non-observers
 
 	m_State = SERVER_STATE_LOADING;
 
 	// Send the final setup state to all clients
-	UpdateGameAttributes(&m_GameAttributes.get());
+	UpdateGameAttributes(&m_GameAttributes);
 
 	// Remove players and observers that are not present when the game starts
 	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end();)
@@ -1248,13 +1360,13 @@ void CNetServerWorker::StartGame()
 
 void CNetServerWorker::UpdateGameAttributes(JS::MutableHandleValue attrs)
 {
-	m_GameAttributes.set(m_ScriptInterface->GetJSRuntime(), attrs);
+	m_GameAttributes = attrs;
 
 	if (!m_Host)
 		return;
 
 	CGameSetupMessage gameSetupMessage(GetScriptInterface());
-	gameSetupMessage.m_Data.set(m_GameAttributes.get());
+	gameSetupMessage.m_Data = m_GameAttributes;
 	Broadcast(&gameSetupMessage);
 }
 
@@ -1289,9 +1401,9 @@ CStrW CNetServerWorker::DeduplicatePlayerName(const CStrW& original)
 	while (true)
 	{
 		bool unique = true;
-		for (size_t i = 0; i < m_Sessions.size(); ++i)
+		for (const CNetServerSession* session : m_Sessions)
 		{
-			if (m_Sessions[i]->GetUserName() == name)
+			if (session->GetUserName() == name)
 			{
 				unique = false;
 				break;
@@ -1318,33 +1430,9 @@ CNetServer::~CNetServer()
 	delete m_Worker;
 }
 
-bool CNetServer::SetupConnection()
+bool CNetServer::SetupConnection(const u16 port)
 {
-	return m_Worker->SetupConnection();
-}
-
-bool CNetServer::KickPlayer(const CStrW& playerName, const bool ban)
-{
-	CScopeLock lock(m_Worker->m_WorkerMutex);
-	return m_Worker->KickPlayer(playerName, ban);
-}
-
-void CNetServer::AssignPlayer(int playerID, const CStr& guid)
-{
-	CScopeLock lock(m_Worker->m_WorkerMutex);
-	m_Worker->m_AssignPlayerQueue.emplace_back(playerID, guid);
-}
-
-void CNetServer::SetPlayerReady(const CStr& guid, int ready)
-{
-	CScopeLock lock(m_Worker->m_WorkerMutex);
-	m_Worker->m_PlayerReadyQueue.emplace_back(guid, ready);
-}
-
-void CNetServer::ClearAllPlayerReady()
-{
-	CScopeLock lock(m_Worker->m_WorkerMutex);
-	m_Worker->m_PlayerResetReadyQueue.push_back(false);
+	return m_Worker->SetupConnection(port);
 }
 
 void CNetServer::StartGame()
