@@ -23,7 +23,7 @@
 
 #include "lib/config2.h"
 #include "ps/ThreadUtil.h"
-#include "scriptinterface/ScriptVal.h"
+#include "scriptinterface/ScriptTypes.h"
 
 #include <vector>
 
@@ -113,32 +113,7 @@ public:
 	 * This function is synchronous (it won't return until the connection is established).
 	 * @return true on success, false on error (e.g. port already in use)
 	 */
-	bool SetupConnection();
-
-	/**
-	 * Call from the GUI to update the player assignments.
-	 * The given GUID will be (re)assigned to the given player ID.
-	 * Any player currently using that ID will be unassigned.
-	 * The changes will be asynchronously propagated to all clients.
-	 */
-	void AssignPlayer(int playerID, const CStr& guid);
-
-	/**
-	 * Call from the GUI to update the player readiness.
-	 * The changes will be asynchronously propagated to all clients.
-	 */
-	void SetPlayerReady(const CStr& guid, int ready);
-
-	/**
-	 * Call from the GUI to set the all player readiness to 0.
-	 * The changes will be asynchronously propagated to all clients.
-	 */
-	void ClearAllPlayerReady();
-
-	/**
-	 * Disconnects a player from gamesetup or session.
-	 */
-	bool KickPlayer(const CStrW& playerName, const bool ban);
+	bool SetupConnection(const u16 port);
 
 	/**
 	 * Call from the GUI to asynchronously notify all clients that they should start loading the game.
@@ -190,7 +165,7 @@ public:
 	/**
 	 * Disconnects a player from gamesetup or session.
 	 */
-	bool KickPlayer(const CStrW& playerName, const bool ban);
+	void KickPlayer(const CStrW& playerName, const bool ban);
 
 	/**
 	 * Send a message to all clients who have completed the full connection process
@@ -209,7 +184,7 @@ private:
 	 * Begin listening for network connections.
 	 * @return true on success, false on error (e.g. port already in use)
 	 */
-	bool SetupConnection();
+	bool SetupConnection(const u16 port);
 
 	/**
 	 * Call from the GUI to update the player assignments.
@@ -270,10 +245,16 @@ private:
 	static bool OnInGame(void* context, CFsmEvent* event);
 	static bool OnChat(void* context, CFsmEvent* event);
 	static bool OnReady(void* context, CFsmEvent* event);
+	static bool OnClearAllReady(void* context, CFsmEvent* event);
+	static bool OnGameSetup(void* context, CFsmEvent* event);
+	static bool OnAssignPlayer(void* context, CFsmEvent* event);
+	static bool OnStartGame(void* context, CFsmEvent* event);
 	static bool OnLoadedGame(void* context, CFsmEvent* event);
 	static bool OnJoinSyncingLoadedGame(void* context, CFsmEvent* event);
 	static bool OnRejoined(void* context, CFsmEvent* event);
+	static bool OnKickPlayer(void* context, CFsmEvent* event);
 	static bool OnDisconnect(void* context, CFsmEvent* event);
+	static bool OnClientPaused(void* context, CFsmEvent* event);
 
 	void CheckGameLoadStatus(CNetServerSession* changedSession);
 
@@ -299,7 +280,7 @@ private:
 	/**
 	 * Stores the most current game attributes.
 	 */
-	DefPersistentRooted<JS::Value> m_GameAttributes;
+	JS::PersistentRootedValue m_GameAttributes;
 
 	int m_AutostartPlayers;
 
@@ -313,9 +294,14 @@ private:
 	CStrW m_ServerName;
 	CStrW m_WelcomeMessage;
 
-	std::vector<CStr> m_BannedIPs;
+	std::vector<u32> m_BannedIPs;
 	std::vector<CStrW> m_BannedPlayers;
 
+	/**
+	 * Holds the GUIDs of all currently paused players.
+	 */
+	std::vector<CStr> m_PausingPlayers;
+
 	u32 m_NextHostID;
 
 	CNetServerTurnManager* m_ServerTurnManager;
@@ -327,7 +313,7 @@ private:
 	 * turn number, to simplify support for rejoining etc.
 	 * TODO: verify this doesn't use too much RAM.
 	 */
-	std::vector<std::vector<CSimulationMessage> > m_SavedCommands;
+	std::vector<std::vector<CSimulationMessage>> m_SavedCommands;
 
 	/**
 	 * The latest copy of the simulation state, received from an existing
@@ -361,10 +347,7 @@ private:
 	bool m_Shutdown; // protected by m_WorkerMutex
 
 	// Queues for messages sent by the game thread:
-	std::vector<std::pair<int, CStr> > m_AssignPlayerQueue; // protected by m_WorkerMutex
 	std::vector<bool> m_StartGameQueue; // protected by m_WorkerMutex
-	std::vector<std::pair<CStr, int> > m_PlayerReadyQueue; // protected by m_WorkerMutex
-	std::vector<bool> m_PlayerResetReadyQueue; // protected by m_WorkerMutex
 	std::vector<std::string> m_GameAttributesQueue; // protected by m_WorkerMutex
 	std::vector<u32> m_TurnLengthQueue; // protected by m_WorkerMutex
 };
