@@ -50,6 +50,8 @@
 #include "maths/MathUtil.h"
 #include "network/NetServer.h"
 #include "network/NetClient.h"
+#include "network/NetMessage.h"
+#include "network/NetMessages.h"
 
 #include "ps/CConsole.h"
 #include "ps/CLogger.h"
@@ -76,6 +78,7 @@
 #include "ps/UserReport.h"
 #include "ps/Util.h"
 #include "ps/VideoMode.h"
+#include "ps/VisualReplay.h"
 #include "ps/World.h"
 
 #include "renderer/Renderer.h"
@@ -699,6 +702,10 @@ static void ShutdownSDL()
 
 void EndGame()
 {
+	if (g_Game && g_Game->IsGameStarted() && !g_Game->IsVisualReplay() &&
+	    g_AtlasGameLoop && !g_AtlasGameLoop->running)
+		VisualReplay::SaveReplayMetadata(g_GUI->GetActiveGUI()->GetScriptInterface().get());
+
 	SAFE_DELETE(g_NetClient);
 	SAFE_DELETE(g_NetServer);
 	SAFE_DELETE(g_Game);
@@ -1168,26 +1175,38 @@ CStr8 LoadSettingsOfScenarioMap(const VfsPath &mapPath)
 }
 
 /*
- * Command line options for autostart (keep synchronized with binaries/system/readme.txt):
+ * Command line options for autostart
+ * (keep synchronized with binaries/system/readme.txt):
+ *
+ * -autostart="TYPEDIR/MAPNAME"    enables autostart and sets MAPNAME;
+ *                                 TYPEDIR is skirmishes, scenarios, or random
+ * -autostart-ai=PLAYER:AI         sets the AI for PLAYER (e.g. 2:petra)
+ * -autostart-aidiff=PLAYER:DIFF   sets the DIFFiculty of PLAYER's AI
+ *                                 (0: sandbox, 5: very hard)
+ * -autostart-aiseed=AISEED        sets the seed used for the AI random
+ *                                 generator (default 0, use -1 for random)
+ * -autostart-civ=PLAYER:CIV       sets PLAYER's civilisation to CIV
+ *                                 (skirmish and random maps only)
+ * -autostart-team=PLAYER:TEAM     sets the team for PLAYER (e.g. 2:2).
  *
- * -autostart="TYPEDIR/MAPNAME"         enables autostart and sets MAPNAME; TYPEDIR is skirmishes, scenarios, or random
- * -autostart-ai=PLAYER:AI              sets the AI for PLAYER (e.g. 2:petra)
- * -autostart-aidiff=PLAYER:DIFF        sets the DIFFiculty of PLAYER's AI (0: sandbox, 5: very hard)
- * -autostart-civ=PLAYER:CIV            sets PLAYER's civilisation to CIV (skirmish and random maps only)
- * -autostart-aiseed=AISEED             sets the seed used for the AI random generator (default 0, use -1 for random)
  * Multiplayer:
- * -autostart-playername=NAME		sets local player NAME (default 'anonymous')
- * -autostart-host                      sets multiplayer host mode
- * -autostart-host-players=NUMBER       sets NUMBER of human players for multiplayer game (default 2)
- * -autostart-client=IP                 sets multiplayer client to join host at given IP address
+ * -autostart-playername=NAME      sets local player NAME (default 'anonymous')
+ * -autostart-host                 sets multiplayer host mode
+ * -autostart-host-players=NUMBER  sets NUMBER of human players for multiplayer
+ *                                 game (default 2)
+ * -autostart-client=IP            sets multiplayer client to join host at
+ *                                 given IP address
  * Random maps only:
- * -autostart-seed=SEED                 sets random map SEED value (default 0, use -1 for random)
- * -autostart-size=TILES                sets random map size in TILES (default 192)
- * -autostart-players=NUMBER            sets NUMBER of players on random map (default 2)
+ * -autostart-seed=SEED            sets random map SEED value
+ *                                 (default 0, use -1 for random)
+ * -autostart-size=TILES           sets random map size in TILES (default 192)
+ * -autostart-players=NUMBER       sets NUMBER of players on random map
+ *                                 (default 2)
  *
  * Examples:
  * 1) "Bob" will host a 2 player game on the Arcadia map:
  * -autostart="scenarios/Arcadia 02" -autostart-host -autostart-host-players=2 -autostart-playername="Bob"
+ * 
  * 2) Load Alpine Lakes random map with random seed, 2 players (Athens and Britons), and player 2 is PetraBot:
  * -autostart="random/alpine_lakes" -autostart-seed=-1 -autostart-players=2 -autostart-civ=1:athen -autostart-civ=2:brit -autostart-ai=2:petra
 */
@@ -1324,7 +1343,40 @@ bool Autostart(const CmdLineArgs& args)
 	scriptInterface.SetProperty(settings, "AISeed", aiseed);
 
 	// Set player data for AIs
-	//		attrs.settings = { PlayerData: [ { AI: ... }, ... ] }:
+	//		attrs.settings = { PlayerData: [ { AI: ... }, ... ] }
+	//		            or = { PlayerData: [ null, { AI: ... }, ... ] } when gaia set
+	int offset = 1;
+	JS::RootedValue player(cx);
+	if (scriptInterface.GetPropertyInt(playerData, 0, &player) && player.isNull())
+		offset = 0;
+
+	// Set teams
+	if (args.Has("autostart-team"))
+	{
+		std::vector<CStr> civArgs = args.GetMultiple("autostart-team");
+		for (size_t i = 0; i < civArgs.size(); ++i)
+		{
+			int playerID = civArgs[i].BeforeFirst(":").ToInt();		
+
+			// Instead of overwriting existing player data, modify the array
+			JS::RootedValue player(cx);
+			if (!scriptInterface.GetPropertyInt(playerData, playerID-offset, &player) || player.isUndefined())
+			{
+				if (mapDirectory == L"skirmishes")
+				{
+					// playerID is certainly bigger than this map player number
+					LOGWARNING("Autostart: Invalid player %d in autostart-team option", playerID);
+					continue;
+				}
+				scriptInterface.Eval("({})", &player);
+			}
+
+			int teamID = civArgs[i].AfterFirst(":").ToInt() - 1;
+			scriptInterface.SetProperty(player, "Team", teamID);
+			scriptInterface.SetPropertyInt(playerData, playerID-offset, player);
+		}
+	}
+
 	if (args.Has("autostart-ai"))
 	{
 		std::vector<CStr> aiArgs = args.GetMultiple("autostart-ai");
@@ -1334,7 +1386,7 @@ bool Autostart(const CmdLineArgs& args)
 
 			// Instead of overwriting existing player data, modify the array
 			JS::RootedValue player(cx);
-			if (!scriptInterface.GetPropertyInt(playerData, playerID-1, &player) || player.isUndefined())
+			if (!scriptInterface.GetPropertyInt(playerData, playerID-offset, &player) || player.isUndefined())
 			{
 				if (mapDirectory == L"scenarios" || mapDirectory == L"skirmishes")
 				{
@@ -1348,7 +1400,7 @@ bool Autostart(const CmdLineArgs& args)
 			CStr name = aiArgs[i].AfterFirst(":");
 			scriptInterface.SetProperty(player, "AI", std::string(name));
 			scriptInterface.SetProperty(player, "AIDiff", 3);
-			scriptInterface.SetPropertyInt(playerData, playerID-1, player);
+			scriptInterface.SetPropertyInt(playerData, playerID-offset, player);
 		}
 	}
 	// Set AI difficulty
@@ -1361,7 +1413,7 @@ bool Autostart(const CmdLineArgs& args)
 
 			// Instead of overwriting existing player data, modify the array
 			JS::RootedValue player(cx);
-			if (!scriptInterface.GetPropertyInt(playerData, playerID-1, &player) || player.isUndefined())
+			if (!scriptInterface.GetPropertyInt(playerData, playerID-offset, &player) || player.isUndefined())
 			{
 				if (mapDirectory == L"scenarios" || mapDirectory == L"skirmishes")
 				{
@@ -1374,7 +1426,7 @@ bool Autostart(const CmdLineArgs& args)
 
 			int difficulty = civArgs[i].AfterFirst(":").ToInt();			
 			scriptInterface.SetProperty(player, "AIDiff", difficulty);
-			scriptInterface.SetPropertyInt(playerData, playerID-1, player);
+			scriptInterface.SetPropertyInt(playerData, playerID-offset, player);
 		}
 	}
 	// Set player data for Civs
@@ -1389,7 +1441,7 @@ bool Autostart(const CmdLineArgs& args)
 
 				// Instead of overwriting existing player data, modify the array
 				JS::RootedValue player(cx);
-				if (!scriptInterface.GetPropertyInt(playerData, playerID-1, &player) || player.isUndefined())
+				if (!scriptInterface.GetPropertyInt(playerData, playerID-offset, &player) || player.isUndefined())
 				{
 					if (mapDirectory == L"skirmishes")
 					{
@@ -1400,9 +1452,9 @@ bool Autostart(const CmdLineArgs& args)
 					scriptInterface.Eval("({})", &player);
 				}
 			
-				CStr name = civArgs[i].AfterFirst(":");			
+				CStr name = civArgs[i].AfterFirst(":");
 				scriptInterface.SetProperty(player, "Civ", std::string(name));
-				scriptInterface.SetPropertyInt(playerData, playerID-1, player);
+				scriptInterface.SetPropertyInt(playerData, playerID-offset, player);
 			}
 		}
 		else
@@ -1438,25 +1490,25 @@ bool Autostart(const CmdLineArgs& args)
 
 		g_NetServer->UpdateGameAttributes(&attrs, scriptInterface);
 
-		bool ok = g_NetServer->SetupConnection();
+		bool ok = g_NetServer->SetupConnection(PS_DEFAULT_PORT);
 		ENSURE(ok);
 
-		g_NetClient = new CNetClient(g_Game);
+		g_NetClient = new CNetClient(g_Game, true);
 		g_NetClient->SetUserName(userName);
-		g_NetClient->SetupConnection("127.0.0.1");
+		g_NetClient->SetupConnection("127.0.0.1", PS_DEFAULT_PORT);
 	}
 	else if (args.Has("autostart-client"))
 	{
 		InitPs(true, L"page_loading.xml", &scriptInterface, mpInitData);
 
-		g_NetClient = new CNetClient(g_Game);
+		g_NetClient = new CNetClient(g_Game, false);
 		g_NetClient->SetUserName(userName);
 
 		CStr ip = args.Get("autostart-client");
 		if (ip.empty())
 			ip = "127.0.0.1";
 
-		bool ok = g_NetClient->SetupConnection(ip);
+		bool ok = g_NetClient->SetupConnection(ip, PS_DEFAULT_PORT);
 		ENSURE(ok);
 	}
 	else
