@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -43,6 +43,8 @@ extern int g_xres, g_yres;
 
 struct CProfileViewerInternals
 {
+	NONCOPYABLE(CProfileViewerInternals); // because of the ofstream
+public:
 	CProfileViewerInternals() {}
 
 	/// Whether the profiling display is currently visible
@@ -60,11 +62,6 @@ struct CProfileViewerInternals
 
 	/// File for saved profile output (reset when the game is restarted)
 	std::ofstream outputStream;
-
-private:
-	// Cannot be copied/assigned, because of the ofstream
-	CProfileViewerInternals(const CProfileViewerInternals& rhs);
-	const CProfileViewerInternals& operator=(const CProfileViewerInternals& rhs);
 };
 
 
