@@ -25,6 +25,7 @@
 #include "lib/res/h_mgr.h"
 #include "lib/tex/tex.h"
 #include "ps/Game.h"
+#include "ps/CLogger.h"
 #include "ps/Loader.h"
 #include "ps/Mod.h"
 #include "ps/Profile.h"
@@ -38,18 +39,7 @@
 #include "simulation2/helpers/SimulationCommand.h"
 
 #include <ctime>
-#include <sstream>
 #include <fstream>
-#include <iomanip>
-
-static std::string Hexify(const std::string& s)
-{
-	std::stringstream str;
-	str << std::hex;
-	for (size_t i = 0; i < s.size(); ++i)
-		str << std::setfill('0') << std::setw(2) << (int)(unsigned char)s[i];
-	return str.str();
-}
 
 CReplayLogger::CReplayLogger(ScriptInterface& scriptInterface) :
 	m_ScriptInterface(scriptInterface), m_Stream(NULL)
@@ -70,7 +60,7 @@ void CReplayLogger::StartGame(JS::MutableHandleValue attribs)
 	m_ScriptInterface.SetProperty(attribs, "engine_version", CStr(engine_version));
 	m_ScriptInterface.SetProperty(attribs, "mods", g_modsLoaded);
 
-	m_Directory = getDateIndexSubdirectory(VisualReplay::GetDirectoryName());
+	m_Directory = createDateIndexSubdirectory(VisualReplay::GetDirectoryName());
 	debug_printf("Writing replay to %s\n", m_Directory.string8().c_str());
 
 	m_Stream = new std::ofstream(OsString(m_Directory / L"commands.txt").c_str(), std::ofstream::out | std::ofstream::trunc);
@@ -168,6 +158,17 @@ void CReplayPlayer::Replay(bool serializationtest, bool ooslog)
 			JS::RootedValue attribs(cx);
 			ENSURE(g_Game->GetSimulation2()->GetScriptInterface().ParseJSON(line, &attribs));
 
+			std::vector<CStr> replayModList;
+			g_Game->GetSimulation2()->GetScriptInterface().GetProperty(attribs, "mods", replayModList);
+
+			for (const CStr& mod : replayModList)
+				if (mod != "user" && std::find(g_modsLoaded.begin(), g_modsLoaded.end(), mod) == g_modsLoaded.end())
+					LOGWARNING("The mod '%s' is required by the replay file, but wasn't passed as an argument!", mod);
+
+			for (const CStr& mod : g_modsLoaded)
+				if (mod != "user" && std::find(replayModList.begin(), replayModList.end(), mod) == replayModList.end())
+					LOGWARNING("The mod '%s' wasn't used when creating this replay file, but was passed as an argument!", mod);
+
 			g_Game->StartGame(&attribs, "");
 
 			// TODO: Non progressive load can fail - need a decent way to handle this
