@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -273,12 +273,14 @@ private:
 		 * occasionally so it can check its timer.
 		 */
 
-		g_Profiler2.RecordRegionEnter("semaphore wait");
-
 		// Wait until the main thread wakes us up
-		while (SDL_SemWait(m_WorkerSem) == 0)
+		while (true)
 		{
-			g_Profiler2.RecordRegionLeave("semaphore wait");
+			g_Profiler2.RecordRegionEnter("semaphore wait");
+
+			ENSURE(SDL_SemWait(m_WorkerSem) == 0);
+
+			g_Profiler2.RecordRegionLeave();
 
 			// Handle shutdown requests as soon as possible
 			if (GetShutdown())
@@ -301,8 +303,6 @@ private:
 					return;
 			}
 		}
-
-		g_Profiler2.RecordRegionLeave("semaphore wait");
 	}
 
 	bool GetEnabled()
