@@ -24,9 +24,7 @@
 
 #include "maths/Fixed.h"
 #include "ScriptTypes.h"
-#include "ScriptVal.h"
 #include "ps/Errors.h"
-#include "ps/Profile.h"
 
 ERROR_GROUP(Scripting);
 ERROR_TYPE(Scripting, SetupFailed);
@@ -98,12 +96,6 @@ public:
 
 	~ScriptInterface();
 
-	/**
-	 * Shut down the JS system to clean up memory. Must only be called when there
-	 * are no ScriptInterfaces alive.
-	 */
-	static void ShutDown();
-
 	struct CxPrivate
 	{
 		ScriptInterface* pScriptInterface; // the ScriptInterface object the current context belongs to
@@ -421,9 +413,9 @@ private:
 			m_Constructor = std::move(other.m_Constructor);
 		}
 
-		DefPersistentRooted<JSObject*>	m_Prototype;
-		JSClass*	m_Class;
-		JSNative 	m_Constructor;
+		JS::PersistentRootedObject m_Prototype;
+		JSClass* m_Class;
+		JSNative m_Constructor;
 	};
 	void Register(const char* name, JSNative fptr, size_t nargs);
 	
@@ -508,7 +500,7 @@ bool ScriptInterface::CallFunctionVoid(JS::HandleValue val, const char* name, co
 	JS::RootedValue jsRet(cx);
 	JS::AutoValueVector argv(cx);
 	argv.resize(1);
-	AssignOrToJSVal(cx, argv.handleAt(0), a0);
+	AssignOrToJSVal(cx, argv[0], a0);
 	return CallFunction_(val, name, argv, &jsRet);
 }
 
@@ -520,8 +512,8 @@ bool ScriptInterface::CallFunctionVoid(JS::HandleValue val, const char* name, co
 	JS::RootedValue jsRet(cx);
 	JS::AutoValueVector argv(cx);
 	argv.resize(2);
-	AssignOrToJSVal(cx, argv.handleAt(0), a0);
-	AssignOrToJSVal(cx, argv.handleAt(1), a1);
+	AssignOrToJSVal(cx, argv[0], a0);
+	AssignOrToJSVal(cx, argv[1], a1);
 	return CallFunction_(val, name, argv, &jsRet);
 }
 
@@ -533,9 +525,9 @@ bool ScriptInterface::CallFunctionVoid(JS::HandleValue val, const char* name, co
 	JS::RootedValue jsRet(cx);
 	JS::AutoValueVector argv(cx);
 	argv.resize(3);
-	AssignOrToJSVal(cx, argv.handleAt(0), a0);
-	AssignOrToJSVal(cx, argv.handleAt(1), a1);
-	AssignOrToJSVal(cx, argv.handleAt(2), a2);
+	AssignOrToJSVal(cx, argv[0], a0);
+	AssignOrToJSVal(cx, argv[1], a1);
+	AssignOrToJSVal(cx, argv[2], a2);
 	return CallFunction_(val, name, argv, &jsRet);
 }
 
