@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 #include "ps/GameSetup/Config.h"
 #include "ps/Profile.h"
+#include "scriptinterface/ScriptEngine.h"
 
 
 void GCSliceCallbackHook(JSRuntime* UNUSED(rt), JS::GCProgress progress, const JS::GCDescription& UNUSED(desc))
@@ -46,7 +47,7 @@ void GCSliceCallbackHook(JSRuntime* UNUSED(rt), JS::GCProgress progress, const J
 	{
 		if (CProfileManager::IsInitialised() && ThreadUtil::IsMainThread())
 			g_Profiler.Stop();
-    	g_Profiler2.RecordRegionLeave("GCSlice");
+    	g_Profiler2.RecordRegionLeave();
 	}
 	else if (progress == JS::GC_CYCLE_BEGIN)
 	{
@@ -58,7 +59,7 @@ void GCSliceCallbackHook(JSRuntime* UNUSED(rt), JS::GCProgress progress, const J
 	{
 		if (CProfileManager::IsInitialised() && ThreadUtil::IsMainThread())
 			g_Profiler.Stop();
-    	g_Profiler2.RecordRegionLeave("GCSlice");
+    	g_Profiler2.RecordRegionLeave();
 	}
 
 	// The following code can be used to print some information aobut garbage collection
@@ -104,61 +105,40 @@ void ScriptRuntime::AddDeferredFinalizationObject(const std::shared_ptr<void>& o
 	m_FinalizationListObjectIdCache.push_back(obj);
 }
 
-bool ScriptRuntime::m_Initialized = false;
-
 ScriptRuntime::ScriptRuntime(shared_ptr<ScriptRuntime> parentRuntime, int runtimeSize, int heapGrowthBytesGCTrigger):
 	m_LastGCBytes(0),
 	m_LastGCCheck(0.0f),
 	m_HeapGrowthBytesGCTrigger(heapGrowthBytesGCTrigger),
 	m_RuntimeSize(runtimeSize)
 {
-	if (!m_Initialized)
-	{
-		ENSURE(JS_Init());
-		m_Initialized = true;
-	}
+	ENSURE(ScriptEngine::IsInitialised() && "The ScriptEngine must be initialized before constructing any ScriptRuntimes!");
 
 	JSRuntime* parentJSRuntime = parentRuntime ? parentRuntime->m_rt : nullptr;
-	m_rt = JS_NewRuntime(runtimeSize, JS_USE_HELPER_THREADS, parentJSRuntime);
+	m_rt = JS_NewRuntime(runtimeSize, JS::DefaultNurseryBytes, parentJSRuntime);
 	ENSURE(m_rt); // TODO: error handling
 
-	if (g_ScriptProfilingEnabled)
-	{
-		// Execute and call hooks are disabled if the runtime debug mode is disabled
-		JS_SetRuntimeDebugMode(m_rt, true);
-
-		// Profiler isn't thread-safe, so only enable this on the main thread
-		if (ThreadUtil::IsMainThread())
-		{
-			if (CProfileManager::IsInitialised())
-			{
-				JS_SetExecuteHook(m_rt, jshook_script, this);
-				JS_SetCallHook(m_rt, jshook_function, this);
-			}
-		}
-	}
-	
 	JS::SetGCSliceCallback(m_rt, GCSliceCallbackHook);
 	JS_SetGCCallback(m_rt, ScriptRuntime::GCCallback, this);
-	
+
 	JS_SetGCParameter(m_rt, JSGC_MAX_MALLOC_BYTES, m_RuntimeSize);
 	JS_SetGCParameter(m_rt, JSGC_MAX_BYTES, m_RuntimeSize);
 	JS_SetGCParameter(m_rt, JSGC_MODE, JSGC_MODE_INCREMENTAL);
-	
+
 	// The whole heap-growth mechanism seems to work only for non-incremental GCs.
 	// We disable it to make it more clear if full GCs happen triggered by this JSAPI internal mechanism.
 	JS_SetGCParameter(m_rt, JSGC_DYNAMIC_HEAP_GROWTH, false);
-	
-	m_dummyContext = JS_NewContext(m_rt, STACK_CHUNK_SIZE);
-	ENSURE(m_dummyContext);
+
+	ScriptEngine::GetSingleton().RegisterRuntime(m_rt);
 }
 
 ScriptRuntime::~ScriptRuntime()
 {
-	JS_DestroyContext(m_dummyContext);
 	JS_SetGCCallback(m_rt, nullptr, nullptr);
 	JS_DestroyRuntime(m_rt);
 	ENSURE(m_FinalizationListObjectIdCache.empty() && "Leak: Removing callback while some objects still aren't finalized!");
+
+	ENSURE(ScriptEngine::IsInitialised() && "The ScriptEngine must be active (initialized and not yet shut down) when destroying a ScriptRuntime!");
+	ScriptEngine::GetSingleton().UnRegisterRuntime(m_rt);
 }
 
 void ScriptRuntime::RegisterContext(JSContext* cx)
@@ -227,7 +207,7 @@ void ScriptRuntime::MaybeIncrementalGC(double delay)
 			
 			// A hack to make sure we never exceed the runtime size because we can't collect the memory
 			// fast enough.
-			if(gcBytes > m_RuntimeSize / 2)
+			if (gcBytes > m_RuntimeSize / 2)
 			{
 				if (JS::IsIncrementalGCInProgress(m_rt))
 				{
@@ -264,7 +244,10 @@ void ScriptRuntime::MaybeIncrementalGC(double delay)
 					printf("Running incremental GC slice \n");
 #endif
 				PrepareContextsForIncrementalGC();
-				JS::IncrementalGC(m_rt, JS::gcreason::REFRESH_FRAME, GCSliceTimeBudget);
+				if (!JS::IsIncrementalGCInProgress(m_rt))
+					JS::StartIncrementalGC(m_rt, GC_NORMAL, JS::gcreason::REFRESH_FRAME, GCSliceTimeBudget);
+				else
+					JS::IncrementalGCSlice(m_rt, JS::gcreason::REFRESH_FRAME, GCSliceTimeBudget);
 			}
 			m_LastGCBytes = gcBytes;
 		}
@@ -275,62 +258,10 @@ void ScriptRuntime::ShrinkingGC()
 {
 	JS_SetGCParameter(m_rt, JSGC_MODE, JSGC_MODE_COMPARTMENT);
 	JS::PrepareForFullGC(m_rt);
-	JS::ShrinkingGC(m_rt, JS::gcreason::REFRESH_FRAME);
+	JS::GCForReason(m_rt, GC_SHRINK, JS::gcreason::REFRESH_FRAME);
 	JS_SetGCParameter(m_rt, JSGC_MODE, JSGC_MODE_INCREMENTAL);
 }
 
-void* ScriptRuntime::jshook_script(JSContext* UNUSED(cx), JSAbstractFramePtr UNUSED(fp), bool UNUSED(isConstructing), bool before, bool* UNUSED(ok), void* closure)
-{
-	if (before)
-		g_Profiler.StartScript("script invocation");
-	else
-		g_Profiler.Stop();
-
-	return closure;
-}
-
-void* ScriptRuntime::jshook_function(JSContext* cx, JSAbstractFramePtr fp, bool UNUSED(isConstructing), bool before, bool* UNUSED(ok), void* closure)
-{
-	JSAutoRequest rq(cx);
-
-	if (!before)
-	{
-		g_Profiler.Stop();
-		return closure;
-	}
-
-	JS::RootedFunction fn(cx, fp.maybeFun());
-	if (!fn)
-	{
-		g_Profiler.StartScript("(function)");
-		return closure;
-	}
-
-	// Try to get the name of non-anonymous functions
-	JS::RootedString name(cx, JS_GetFunctionId(fn));
-	if (name)
-	{
-		char* chars = JS_EncodeString(cx, name);
-		if (chars)
-		{
-			g_Profiler.StartScript(StringFlyweight(chars).get().c_str());
-			JS_free(cx, chars);
-			return closure;
-		}
-	}
-
-	// No name - use fileName and line instead
-	JS::AutoFilename fileName;
-	unsigned lineno;
-	JS::DescribeScriptedCaller(cx, &fileName, &lineno);
-
-	std::stringstream ss;
-	ss << "(" << fileName.get() << ":" << lineno << ")";
-	g_Profiler.StartScript(StringFlyweight(ss.str()).get().c_str());
-
-	return closure;
-}
-
 void ScriptRuntime::PrepareContextsForIncrementalGC()
 {
 	for (JSContext* const& ctx : m_Contexts)
