@@ -46,15 +46,6 @@
 
 #include <iomanip>
 
-static std::string Hexify(const std::string& s) // TODO: shouldn't duplicate this function in so many places
-{
-	std::stringstream str;
-	str << std::hex;
-	for (size_t i = 0; i < s.size(); ++i)
-		str << std::setfill('0') << std::setw(2) << (int)(unsigned char)s[i];
-	return str.str();
-}
-
 class CSimulation2Impl
 {
 public:
@@ -76,10 +67,11 @@ public:
 			CFG_GET_VAL("serializationtest", m_EnableSerializationTest);
 		}
 
-		m_OOSLogPath = getDateIndexSubdirectory(psLogDir() / "oos_logs");
-
 		if (m_EnableOOSLog)
+		{
+			m_OOSLogPath = createDateIndexSubdirectory(psLogDir() / "oos_logs");
 			debug_printf("Writing ooslogs to %s\n", m_OOSLogPath.string8().c_str());
+		}
 	}
 
 	~CSimulation2Impl()
@@ -296,7 +288,7 @@ void CSimulation2Impl::ReportSerializationFailure(
 	SerializationTestState* primaryStateBefore, SerializationTestState* primaryStateAfter,
 	SerializationTestState* secondaryStateBefore, SerializationTestState* secondaryStateAfter)
 {
-	const OsPath path = getDateIndexSubdirectory(psLogDir() / "serializationtest");
+	const OsPath path = createDateIndexSubdirectory(psLogDir() / "serializationtest");
 	debug_printf("Writing serializationtest-data to %s\n", path.string8().c_str());
 
 	// Clean up obsolete files from previous runs
@@ -479,8 +471,11 @@ void CSimulation2Impl::UpdateComponents(CSimContext& simContext, fixed turnLengt
 
 	CComponentManager& componentManager = simContext.GetComponentManager();
 
-	CMessageTurnStart msgTurnStart;
-	componentManager.BroadcastMessage(msgTurnStart);
+	{
+		PROFILE2("Sim - Update Start");
+		CMessageTurnStart msgTurnStart;
+		componentManager.BroadcastMessage(msgTurnStart);
+	}
 
 	CmpPtr<ICmpPathfinder> cmpPathfinder(simContext, SYSTEM_ENTITY);
 	if (cmpPathfinder)
@@ -504,6 +499,7 @@ void CSimulation2Impl::UpdateComponents(CSimContext& simContext, fixed turnLengt
 
 	// Send all the update phases
 	{
+		PROFILE2("Sim - Update");
 		CMessageUpdate msgUpdate(turnLengthFixed);
 		componentManager.BroadcastMessage(msgUpdate);
 	}
@@ -517,10 +513,12 @@ void CSimulation2Impl::UpdateComponents(CSimContext& simContext, fixed turnLengt
 		cmpPathfinder->ProcessSameTurnMoves();
 
 	{
+		PROFILE2("Sim - Motion Unit");
 		CMessageUpdate_MotionUnit msgUpdate(turnLengthFixed);
 		componentManager.BroadcastMessage(msgUpdate);
 	}
 	{
+		PROFILE2("Sim - Update Final");
 		CMessageUpdate_Final msgUpdate(turnLengthFixed);
 		componentManager.BroadcastMessage(msgUpdate);
 	}
@@ -555,6 +553,12 @@ void CSimulation2Impl::DumpState()
 	const OsPath path = m_OOSLogPath / name.str();
 	std::ofstream file (OsString(path).c_str(), std::ofstream::out | std::ofstream::trunc);
 
+	if (!DirectoryExists(m_OOSLogPath))
+	{
+		LOGWARNING("OOS-log directory %s was deleted, creating it again.", m_OOSLogPath.string8().c_str());
+		CreateDirectories(m_OOSLogPath, 0700);
+	}
+
 	file << "State hash: " << std::hex;
 	std::string hashRaw;
 	m_ComponentManager.ComputeStateHash(hashRaw, false);
@@ -590,6 +594,8 @@ void CSimulation2::EnableOOSLog()
 		return;
 
 	m->m_EnableOOSLog = true;
+	m->m_OOSLogPath = createDateIndexSubdirectory(psLogDir() / "oos_logs");
+
 	debug_printf("Writing ooslogs to %s\n", m->m_OOSLogPath.string8().c_str());
 }
 
@@ -598,11 +604,6 @@ void CSimulation2::EnableSerializationTest()
 	m->m_EnableSerializationTest = true;
 }
 
-void CSimulation2::SetCurrentDisplayedPlayer(int playerID)
-{
-	m->m_SimContext.SetCurrentDisplayedPlayer(playerID);
-}
-
 entity_id_t CSimulation2::AddEntity(const std::wstring& templateName)
 {
 	return m->m_ComponentManager.AddEntity(templateName, m->m_ComponentManager.AllocateNewEntity());
@@ -653,7 +654,7 @@ const CSimulation2::InterfaceListUnordered& CSimulation2::GetEntitiesWithInterfa
 	return m->m_ComponentManager.GetEntitiesWithInterfaceUnordered(iid);
 }
 
-CSimContext& CSimulation2::GetSimContext() const
+const CSimContext& CSimulation2::GetSimContext() const
 {
 	return m->m_SimContext;
 }
@@ -738,6 +739,11 @@ JS::Value CSimulation2::GetInitAttributes()
 	return m->m_InitAttributes.get();
 }
 
+void CSimulation2::GetInitAttributes(JS::MutableHandleValue ret)
+{
+	ret.set(m->m_InitAttributes);
+}
+
 void CSimulation2::SetMapSettings(const std::string& settings)
 {
 	m->m_ComponentManager.GetScriptInterface().ParseJSON(settings, &m->m_MapSettings);
@@ -746,6 +752,12 @@ void CSimulation2::SetMapSettings(const std::string& settings)
 void CSimulation2::SetMapSettings(JS::HandleValue settings)
 {
 	m->m_MapSettings = settings;
+
+	u32 seed = 0;
+	if (!m->m_ComponentManager.GetScriptInterface().GetProperty(m->m_MapSettings, "Seed", seed))
+		LOGWARNING("CSimulation2::SetInitAttributes: No seed value specified - using %d", seed);
+
+	m->m_ComponentManager.SetRNGSeed(seed);
 }
 
 std::string CSimulation2::GetMapSettingsString()
