@@ -57,8 +57,6 @@ public:
 	void EnableOOSLog();
 	void EnableSerializationTest();
 
-	void SetCurrentDisplayedPlayer(int playerID);
-
 	/**
 	 * Load all scripts in the specified directory (non-recursively),
 	 * so they can register new component types and functions. This
@@ -105,16 +103,19 @@ public:
 	 * Get the data passed to SetInitAttributes.
 	 */
 	JS::Value GetInitAttributes();
+	void GetInitAttributes(JS::MutableHandleValue ret);
 
 	/**
 	 * Set the initial map settings (as a UTF-8-encoded JSON string),
 	 * which will be used to set up the simulation state.
+	 * Called from atlas.
 	 */
 	void SetMapSettings(const std::string& settings);
 
 	/**
 	 * Set the initial map settings, which will be used
 	 * to set up the simulation state.
+	 * Called from MapReader (for all map-types).
 	 */
 	void SetMapSettings(JS::HandleValue settings);
 
@@ -219,7 +220,7 @@ public:
 	 */
 	const InterfaceListUnordered& GetEntitiesWithInterfaceUnordered(int iid);
 
-	CSimContext& GetSimContext() const;
+	const CSimContext& GetSimContext() const;
 	ScriptInterface& GetScriptInterface() const;
 
 	bool ComputeStateHash(std::string& outHash, bool quick);
