@@ -28,13 +28,14 @@
 #include "lib/allocators/shared_ptr.h"
 #include "ps/CLogger.h"
 #include "ps/Filesystem.h"
+#include "ps/Profile.h"
 #include "ps/Util.h"
 #include "simulation2/components/ICmpAIInterface.h"
 #include "simulation2/components/ICmpCommandQueue.h"
 #include "simulation2/components/ICmpObstructionManager.h"
 #include "simulation2/components/ICmpRangeManager.h"
 #include "simulation2/components/ICmpTemplateManager.h"
-#include "simulation2/components/ICmpTechnologyTemplateManager.h"
+#include "simulation2/components/ICmpDataTemplateManager.h"
 #include "simulation2/components/ICmpTerritoryManager.h"
 #include "simulation2/helpers/LongPathfinder.h"
 #include "simulation2/serialization/DebugSerializer.h"
@@ -224,7 +225,6 @@ public:
 		m_ScriptInterface->RegisterFunction<void, CAIWorker::ForceGC>("ForceGC");
 
 		m_ScriptInterface->RegisterFunction<JS::Value, JS::HandleValue, JS::HandleValue, pass_class_t, CAIWorker::ComputePath>("ComputePath");
-		m_ScriptInterface->RegisterFunction<JS::Value, pass_class_t, CAIWorker::GetConnectivityGrid>("GetConnectivityGrid");
 
 		m_ScriptInterface->RegisterFunction<void, std::wstring, std::vector<u32>, u32, u32, u32, CAIWorker::DumpImage>("DumpImage");
 	}
@@ -297,6 +297,7 @@ public:
 		ENSURE(pCxPrivate->pCBData);
 		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
 		JSContext* cx(self->m_ScriptInterface->GetContext());
+		JSAutoRequest rq(cx);
 
 		CFixedVector2D pos, goalPos;
 		std::vector<CFixedVector2D> waypoints;
@@ -321,17 +322,6 @@ public:
 			waypoints.emplace_back(wp.x, wp.z);
 	}
 
-	static JS::Value GetConnectivityGrid(ScriptInterface::CxPrivate* pCxPrivate, pass_class_t passClass)
-	{
-		ENSURE(pCxPrivate->pCBData);
-		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
-		JSContext* cx(self->m_ScriptInterface->GetContext());
-
-		JS::RootedValue retVal(cx);
-		self->m_ScriptInterface->ToJSVal<Grid<u16> >(cx, &retVal, self->m_LongPathfinder.GetConnectivityGrid(passClass));
-		return retVal;
-	}
-
 	static void ForceGC(ScriptInterface::CxPrivate* pCxPrivate)
 	{
 		PROFILE3("AI compute GC");
@@ -511,32 +501,78 @@ public:
 
 		return true;
 	}
-	void StartComputation(const shared_ptr<ScriptInterface::StructuredClone>& gameState, 
-		const Grid<NavcellData>& passabilityMap, const GridUpdateInformation& dirtinessInformations,
-		const Grid<u8>& territoryMap, bool territoryMapDirty, 
+
+	void UpdateGameState(const shared_ptr<ScriptInterface::StructuredClone>& gameState)
+	{
+		ENSURE(m_CommandsComputed);
+		m_GameState = gameState;
+	}
+
+	void UpdatePathfinder(const Grid<NavcellData>& passabilityMap, bool globallyDirty, const Grid<u8>& dirtinessGrid, bool justDeserialized,
 		const std::map<std::string, pass_class_t>& nonPathfindingPassClassMasks, const std::map<std::string, pass_class_t>& pathfindingPassClassMasks)
 	{
 		ENSURE(m_CommandsComputed);
+		bool dimensionChange = m_PassabilityMap.m_W != passabilityMap.m_W || m_PassabilityMap.m_H != passabilityMap.m_H;
+
+		m_PassabilityMap = passabilityMap;
+		if (globallyDirty)
+			m_LongPathfinder.Reload(&m_PassabilityMap, nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+		else
+			m_LongPathfinder.Update(&m_PassabilityMap, dirtinessGrid);
 
-		m_GameState = gameState;
 		JSContext* cx = m_ScriptInterface->GetContext();
-
-		if (dirtinessInformations.dirty)
-		{
-			m_PassabilityMap = passabilityMap;
-			if (dirtinessInformations.globallyDirty)
-				m_LongPathfinder.Reload(&m_PassabilityMap, nonPathfindingPassClassMasks, pathfindingPassClassMasks);
-			else
-				m_LongPathfinder.Update(&m_PassabilityMap, dirtinessInformations.dirtinessGrid);
+		if (dimensionChange || justDeserialized)
 			ScriptInterface::ToJSVal(cx, &m_PassabilityMapVal, m_PassabilityMap);
-		}
-
-		if (territoryMapDirty)
+		else
 		{
-			m_TerritoryMap = territoryMap;
-			ScriptInterface::ToJSVal(cx, &m_TerritoryMapVal, m_TerritoryMap);
-		}
+			// Avoid a useless memory reallocation followed by a garbage collection.
+			JSAutoRequest rq(cx);
 
+			JS::RootedObject mapObj(cx, &m_PassabilityMapVal.toObject());
+			JS::RootedValue mapData(cx);
+			ENSURE(JS_GetProperty(cx, mapObj, "data", &mapData));
+			JS::RootedObject dataObj(cx, &mapData.toObject());
+
+			u32 length = 0;
+			ENSURE(JS_GetArrayLength(cx, dataObj, &length));
+			u32 nbytes = (u32)(length * sizeof(NavcellData));
+
+			JS::AutoCheckCannotGC nogc;
+			memcpy((void*)JS_GetUint16ArrayData(dataObj, nogc), m_PassabilityMap.m_Data, nbytes);
+		}		
+	}
+
+	void UpdateTerritoryMap(const Grid<u8>& territoryMap)
+	{
+		ENSURE(m_CommandsComputed);
+		bool dimensionChange = m_TerritoryMap.m_W != territoryMap.m_W || m_TerritoryMap.m_H != territoryMap.m_H;
+
+		m_TerritoryMap = territoryMap;
+		
+		JSContext* cx = m_ScriptInterface->GetContext();
+		if (dimensionChange)
+			ScriptInterface::ToJSVal(cx, &m_TerritoryMapVal, m_TerritoryMap);
+		else
+		{
+			// Avoid a useless memory reallocation followed by a garbage collection.
+			JSAutoRequest rq(cx);
+
+			JS::RootedObject mapObj(cx, &m_TerritoryMapVal.toObject());
+			JS::RootedValue mapData(cx);
+			ENSURE(JS_GetProperty(cx, mapObj, "data", &mapData));
+			JS::RootedObject dataObj(cx, &mapData.toObject());
+
+			u32 length = 0;
+			ENSURE(JS_GetArrayLength(cx, dataObj, &length));
+			u32 nbytes = (u32)(length * sizeof(u8));
+
+			JS::AutoCheckCannotGC nogc;
+			memcpy((void*)JS_GetUint8ArrayData(dataObj, nogc), m_TerritoryMap.m_Data, nbytes);
+		}
+	}
+
+	void StartComputation()
+	{
 		m_CommandsComputed = false;
 	}
 
@@ -609,6 +645,9 @@ public:
 
 	void SerializeState(ISerializer& serializer)
 	{
+		if (m_Players.empty())
+			return;
+
 		JSContext* cx = m_ScriptInterface->GetContext();
 		JSAutoRequest rq(cx);
 
@@ -665,6 +704,12 @@ public:
 
 	void Deserialize(std::istream& stream, u32 numAis)
 	{
+		m_PlayerMetadata.clear();
+		m_Players.clear();
+
+		if (numAis == 0)
+			return;
+
 		JSContext* cx = m_ScriptInterface->GetContext();
 		JSAutoRequest rq(cx);
 
@@ -672,9 +717,6 @@ public:
 
 		CStdDeserializer deserializer(*m_ScriptInterface, stream);
 
-		m_PlayerMetadata.clear();
-		m_Players.clear();
-
 		std::string rngString;
 		std::stringstream rngStream;
 		deserializer.StringASCII("rng", rngString, 0, 32);
@@ -763,13 +805,17 @@ public:
 		// Require unique prototype and name (for reverse lookup)
 		// TODO: this is yucky - see comment in Deserialize()
 		ENSURE(proto.isObject() && "A serializable prototype has to be an object!");
-		JS::RootedObject obj(m_ScriptInterface->GetContext(), &proto.toObject());
+
+		JSContext* cx = m_ScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
+
+		JS::RootedObject obj(cx, &proto.toObject());
 		if (m_SerializablePrototypes->has(obj) || m_DeserializablePrototypes.find(name) != m_DeserializablePrototypes.end())
 		{
 			LOGERROR("RegisterSerializablePrototype called with same prototype multiple times: p=%p n='%s'", (void *)obj.get(), utf8_from_wstring(name));
 			return;
 		}
-		m_SerializablePrototypes->add(m_ScriptInterface->GetContext(), obj, name);
+		m_SerializablePrototypes->add(cx, obj, name);
 		m_DeserializablePrototypes[name] = JS::Heap<JSObject*>(obj);
 	}
 
@@ -782,9 +828,9 @@ private:
 	void TraceMember(JSTracer *trc)
 	{
 		for (std::pair<const std::wstring, JS::Heap<JSObject*>>& prototype : m_DeserializablePrototypes)
-			JS_CallHeapObjectTracer(trc, &prototype.second, "CAIWorker::m_DeserializablePrototypes");
+			JS_CallObjectTracer(trc, &prototype.second, "CAIWorker::m_DeserializablePrototypes");
 		for (std::pair<const VfsPath, JS::Heap<JS::Value>>& metadata : m_PlayerMetadata)
-			JS_CallHeapValueTracer(trc, &metadata.second, "CAIWorker::m_PlayerMetadata");
+			JS_CallValueTracer(trc, &metadata.second, "CAIWorker::m_PlayerMetadata");
 	}
 
 	void LoadMetadata(const VfsPath& path, JS::MutableHandleValue out)
@@ -979,12 +1025,12 @@ public:
 		JSAutoRequest rq(cx);
 
 		// load the technology templates
-		CmpPtr<ICmpTechnologyTemplateManager> cmpTechTemplateManager(GetSystemEntity());
-		ENSURE(cmpTechTemplateManager);
+		CmpPtr<ICmpDataTemplateManager> cmpDataTemplateManager(GetSystemEntity());
+		ENSURE(cmpDataTemplateManager);
 
 		// Get the game state from AIInterface
 		JS::RootedValue techTemplates(cx);
-		cmpTechTemplateManager->GetAllTechs(&techTemplates);
+		cmpDataTemplateManager->GetAllTechs(&techTemplates);
 
 		m_Worker.RegisterTechTemplates(scriptInterface.WriteStructuredClone(techTemplates));
 		m_Worker.TryLoadSharedComponent(true);
@@ -1051,37 +1097,40 @@ public:
 			cmpAIInterface->GetFullRepresentation(&state, false);
 		else
 			cmpAIInterface->GetRepresentation(&state);
+		LoadPathfinderClasses(state); // add the pathfinding classes to it
 
-		// Get the passability data
-		Grid<NavcellData> dummyGrid;
-		const Grid<NavcellData>* passabilityMap = &dummyGrid;
+		// Update the game state
+		m_Worker.UpdateGameState(scriptInterface.WriteStructuredClone(state));
+
+		// Update the pathfinding data
 		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (cmpPathfinder)
-			passabilityMap = &cmpPathfinder->GetPassabilityGrid();
-
-		GridUpdateInformation dirtinessInformations = cmpPathfinder->GetDirtinessData();
-
-		// Get the territory data
-		// Since getting the territory grid can trigger a recalculation, we check NeedUpdate first
-		bool territoryMapDirty = false;
-		Grid<u8> dummyGrid2;
-		const Grid<u8>* territoryMap = &dummyGrid2;
-		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSystemEntity());
-		if (cmpTerritoryManager && cmpTerritoryManager->NeedUpdate(&m_TerritoriesDirtyID))
 		{
-			territoryMap = &cmpTerritoryManager->GetTerritoryGrid();
-			territoryMapDirty = true;
+			GridUpdateInformation dirtinessInformations = cmpPathfinder->GetDirtinessData();
+
+			if (dirtinessInformations.dirty || m_JustDeserialized)
+			{
+				const Grid<NavcellData>& passabilityMap = cmpPathfinder->GetPassabilityGrid();
+
+				std::map<std::string, pass_class_t> nonPathfindingPassClassMasks, pathfindingPassClassMasks;
+				cmpPathfinder->GetPassabilityClasses(nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+
+				m_Worker.UpdatePathfinder(passabilityMap,
+					dirtinessInformations.globallyDirty, dirtinessInformations.dirtinessGrid, m_JustDeserialized,
+					nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+			}
 		}
 
-		LoadPathfinderClasses(state);
-		std::map<std::string, pass_class_t> nonPathfindingPassClassMasks, pathfindingPassClassMasks;
-		if (cmpPathfinder)
-			cmpPathfinder->GetPassabilityClasses(nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+		// Update the territory data
+		// Since getting the territory grid can trigger a recalculation, we check NeedUpdate first
+		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSystemEntity());
+		if (cmpTerritoryManager && (cmpTerritoryManager->NeedUpdate(&m_TerritoriesDirtyID) || m_JustDeserialized))
+		{
+			const Grid<u8>& territoryMap = cmpTerritoryManager->GetTerritoryGrid();
+			m_Worker.UpdateTerritoryMap(territoryMap);
+		}
 
-		m_Worker.StartComputation(scriptInterface.WriteStructuredClone(state), 
-			*passabilityMap, dirtinessInformations,
-			*territoryMap, territoryMapDirty,
-			nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+		m_Worker.StartComputation();
 
 		m_JustDeserialized = false;
 	}
