@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -135,22 +135,26 @@ struct SerializeShortRequest
 	}
 };
 
-void CCmpPathfinder::Serialize(ISerializer& serialize)
+template<typename S>
+void CCmpPathfinder::SerializeCommon(S& serialize)
 {
 	SerializeVector<SerializeLongRequest>()(serialize, "long requests", m_AsyncLongPathRequests);
 	SerializeVector<SerializeShortRequest>()(serialize, "short requests", m_AsyncShortPathRequests);
 	serialize.NumberU32_Unbounded("next ticket", m_NextAsyncTicket);
 	serialize.NumberU16_Unbounded("same turn moves count", m_SameTurnMovesCount);
+	serialize.NumberU16_Unbounded("map size", m_MapSize);
+}
+
+void CCmpPathfinder::Serialize(ISerializer& serialize)
+{
+	SerializeCommon(serialize);
 }
 
 void CCmpPathfinder::Deserialize(const CParamNode& paramNode, IDeserializer& deserialize)
 {
 	Init(paramNode);
 
-	SerializeVector<SerializeLongRequest>()(deserialize, "long requests", m_AsyncLongPathRequests);
-	SerializeVector<SerializeShortRequest>()(deserialize, "short requests", m_AsyncShortPathRequests);
-	deserialize.NumberU32_Unbounded("next ticket", m_NextAsyncTicket);
-	deserialize.NumberU16_Unbounded("same turn moves count", m_SameTurnMovesCount);
+	SerializeCommon(deserialize);
 }
 
 void CCmpPathfinder::HandleMessage(const CMessage& msg, bool UNUSED(global))
@@ -684,6 +688,7 @@ u32 CCmpPathfinder::ComputeShortPathAsync(entity_pos_t x0, entity_pos_t z0, enti
 
 void CCmpPathfinder::FinishAsyncRequests()
 {
+	PROFILE2("Finish Async Requests");
 	// Save the request queue in case it gets modified while iterating
 	std::vector<AsyncLongPathRequest> longRequests;
 	m_AsyncLongPathRequests.swap(longRequests);
@@ -702,6 +707,7 @@ void CCmpPathfinder::FinishAsyncRequests()
 
 void CCmpPathfinder::ProcessLongRequests(const std::vector<AsyncLongPathRequest>& longRequests)
 {
+	PROFILE2("Process Long Requests");
 	for (size_t i = 0; i < longRequests.size(); ++i)
 	{
 		const AsyncLongPathRequest& req = longRequests[i];
@@ -714,6 +720,7 @@ void CCmpPathfinder::ProcessLongRequests(const std::vector<AsyncLongPathRequest>
 
 void CCmpPathfinder::ProcessShortRequests(const std::vector<AsyncShortPathRequest>& shortRequests)
 {
+	PROFILE2("Process Short Requests");
 	for (size_t i = 0; i < shortRequests.size(); ++i)
 	{
 		const AsyncShortPathRequest& req = shortRequests[i];
@@ -788,6 +795,8 @@ bool CCmpPathfinder::CheckMovement(const IObstructionTestFilter& filter,
 	entity_pos_t x0, entity_pos_t z0, entity_pos_t x1, entity_pos_t z1, entity_pos_t r,
 	pass_class_t passClass)
 {
+	PROFILE2_IFSPIKE("Check Movement", 0.001);
+
 	// Test against obstructions first. filter may discard pathfinding-blocking obstructions.
 	// Use more permissive version of TestLine to allow unit-unit collisions to overlap slightly.
 	// This makes movement smoother and more natural for units, overall.
